package org.brynhild.graduation.activity.admin

import android.app.Activity
import android.app.AlertDialog
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.os.Message
import android.util.Log
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuItem
import android.widget.Toast
import androidx.recyclerview.widget.GridLayoutManager
import com.bumptech.glide.Glide
import com.example.devicemanager.view.ConfirmPopupWindow
import com.google.gson.Gson
import org.brynhild.graduation.R
import org.brynhild.graduation.activity.common.BaseActivity
import org.brynhild.graduation.common.callback.RecyclerViewCallback
import org.brynhild.graduation.common.config.LoginConfiguration
import org.brynhild.graduation.common.config.MyApplication
import org.brynhild.graduation.common.transfer.Result
import org.brynhild.graduation.common.transfer.dto.FileInfo
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.user.*
import org.brynhild.graduation.common.utils.FileUploadUtil
import org.brynhild.graduation.common.utils.JsonUtils
import org.brynhild.graduation.common.utils.MultipleSelectHandler
import org.brynhild.graduation.common.utils.Uri2PathUtil
import org.brynhild.graduation.databinding.ActivityAcademyDetailInfoBinding
import org.brynhild.graduation.databinding.AdminCreateMajorDialogBinding
import org.brynhild.graduation.network.adapter.MajorAdapter
import org.brynhild.graduation.network.entiity.Academy
import org.brynhild.graduation.network.entiity.Course
import org.brynhild.graduation.network.entiity.Major
import org.brynhild.graduation.service.http.AdminService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import java.io.File
import java.io.IOException

class ActivityDetailInfoActivity : BaseActivity() {
    private lateinit var binding: ActivityAcademyDetailInfoBinding
    private lateinit var course: Course
    private val updateAccountAvatar = 2
    private val refreshMajorList = 0

    val handler: Handler = Handler {
        when (it.what) {
            updateAccountAvatar -> {
                val path = it.obj as String
                Glide.with(this).load(path).into(binding.iconImage)
                val service = ServiceCreator.create(AdminService::class.java)
                val data = AdminModifyAcademy(course.id, null, null, path, null)
                service.modifyAcademy(LoginConfiguration.userInfo!!.token, data)
                    .enqueue(object : Callback<Result> {
                        override fun onResponse(call: Call<Result>, response: Response<Result>) {
                            ResponseHandler.handle(response, { ret ->
                                ResponseHandler.showMessage(ret)
                            }, { ret ->
                                ResponseHandler.showMessage(ret)
                            })
                        }

                        override fun onFailure(call: Call<Result>, t: Throwable) {
                            Toast.makeText(MyApplication.context, "请求失败", Toast.LENGTH_SHORT).show()
                        }
                    })
            }
            refreshMajorList -> {
                loadMajorList()
            }
        }
        false
    }

    private fun loadMajorList() {
        val service = ServiceCreator.create(AdminService::class.java)
        val data = QueryMajorRequest(null, null, course.id)
        Log.d("Admin", "查询专业列表：${data}")
        service.queryMajor(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "AddDeviceActivity",
                            "flag is ${result.flag},message is ${result.message},data is ${result.data} in academy"
                        )
                        val majorArrayList = ArrayList<Major>()
                        val gson = Gson()
                        for (item in result.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val major = JsonUtils.fromJson<Major>(json)
                            if (major != null) {
                                majorArrayList.add(major)
                            }
                        }
                        val adapter = MajorAdapter(this@ActivityDetailInfoActivity, majorArrayList)

                        binding.recyclerView.adapter = MultipleSelectHandler.createHandler(
                            this@ActivityDetailInfoActivity,
                            adapter,
                            object :
                                RecyclerViewCallback {
                                override fun execute(selected: List<Int>) {
                                    val builder = StringBuilder()
                                    println(selected)
                                    var first = true

                                    val rmList = ArrayList<Long>()
                                    for (index in selected) {
                                        rmList.add(majorArrayList[index].id)
                                        if (!first) {
                                            builder.append("、")
                                        } else {
                                            first = false
                                        }
                                        builder.append(majorArrayList[index].name)
                                    }
                                    ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@ActivityDetailInfoActivity)
                                        .setContent("以下专业将被删除：${builder}")
                                        .setCancelText("取消删除")
                                        .setEnsureText("确定删除")
                                        .setEnsureListener {

                                            val service =
                                                ServiceCreator.create(AdminService::class.java)
                                            val rmData = AdminDeleteMajor(rmList)
                                            println("send data with:${Gson().toJson(rmData)}")
                                            service.deleteMajor(
                                                LoginConfiguration.userInfo!!.token,
                                                rmData
                                            ).enqueue(object : Callback<Result> {
                                                override fun onResponse(
                                                    call: Call<Result>,
                                                    response: Response<Result>
                                                ) {
                                                    ResponseHandler.handle(response, {
                                                        handler.sendEmptyMessage(refreshMajorList)
                                                    }, {
                                                        ResponseHandler.showMessage(it)
                                                    })
                                                }

                                                override fun onFailure(
                                                    call: Call<Result>,
                                                    t: Throwable
                                                ) {
                                                    t.printStackTrace()
                                                }
                                            })

                                        }
                                        .build()
                                        .show()
                                }
                            })
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityAcademyDetailInfoBinding.inflate(LayoutInflater.from(this))
        setContentView(binding.root)
        setSupportActionBar(binding.toolBar)
        val layoutManager = GridLayoutManager(this, 1)
        binding.recyclerView.layoutManager = layoutManager

        val gson = Gson()
        course = gson.fromJson(intent.getStringExtra("course"), Course::class.java)

//        setTableData(course)

        bindOnClick()

        loadMajorList()
    }

    private fun bindOnClick() {
        binding.backButton.setOnClickListener {
            finish()
        }

        binding.iconImage.setOnClickListener {
            val intent = Intent(Intent.ACTION_OPEN_DOCUMENT)
            intent.addCategory(Intent.CATEGORY_OPENABLE)
            intent.type = "image/*"
            startActivityForResult(intent, updateAccountAvatar)
        }
    }

    private fun setTableData(academy: Academy) {
        binding.no.text = academy.no
        binding.liable.text = "${academy.liable}"
        binding.title.text = academy.name

        Glide.with(this).load(academy.avatar ?: R.drawable.nav_academy).into(binding.iconImage)
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.admin_academy_detail_info_options, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            R.id.admin_academy_delete -> {
                ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@ActivityDetailInfoActivity)
                    .setContent("确定删除学院：${course.name}吗")
                    .setCancelText("取消删除")
                    .setEnsureText("确定删除")
                    .setEnsureListener {
                        val service = ServiceCreator.create(AdminService::class.java)
                        val rmList = ArrayList<Long>()
                        rmList.add(course.id)
                        val rmData = AdminDeleteAcademy(rmList)
                        service.deleteAcademy(LoginConfiguration.userInfo!!.token, rmData)
                            .enqueue(object :
                                Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        finish()
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .build()
                    .show()


            }
            R.id.admin_major_add -> {
                val dialogLayout = AdminCreateMajorDialogBinding.inflate(LayoutInflater.from(this))

                AlertDialog.Builder(this)
                    .setTitle("添加专业")
                    .setView(dialogLayout.root)
                    .setPositiveButton("确认添加") { _, _ ->
                        val service = ServiceCreator.create(AdminService::class.java)
                        val data = AdminAddMajor(
                            dialogLayout.name.text.toString(),
                            dialogLayout.code.text.toString(),
                            course.id
                        )
                        service.addMajor(LoginConfiguration.userInfo!!.token, data)
                            .enqueue(object : Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        handler.sendEmptyMessage(refreshMajorList)
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .setNegativeButton("取消添加") { _, _ ->

                    }
                    .show()
            }
        }
        return true
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (requestCode) {
            updateAccountAvatar -> {
                if (resultCode == Activity.RESULT_OK && data != null) {
                    val uri = data.data
                    if (Uri2PathUtil.getRealPathFromUri(this, uri) != null) {
                        //从uri得到绝对路径，并获取到file文件
                        val file = File(Uri2PathUtil.getRealPathFromUri(this, uri))
                        FileUploadUtil.uploadFile(file, object : okhttp3.Callback {
                            override fun onFailure(call: okhttp3.Call, e: IOException) {
                                println("failed")
                            }

                            override fun onResponse(
                                call: okhttp3.Call,
                                response: okhttp3.Response
                            ) {
                                if (response.isSuccessful) {
                                    val body = response.body()?.string()
                                    val gson = Gson()
                                    val result = gson.fromJson(body, Result::class.java)
                                    println(result)
                                    var json = gson.toJson(result.data)
                                    val fileInfoList = JsonUtils.fromJson2List<FileInfo>(json)
                                    if (fileInfoList?.size == 1) {
                                        val message = Message()
                                        message.what = updateAccountAvatar
                                        json = gson.toJson(fileInfoList[0])
                                        val info = JsonUtils.fromJson<FileInfo>(json)
                                        println(info)
                                        if (info != null) {
                                            message.obj = info.path
                                            handler.sendMessage(message)
                                        }
                                    }
                                }
                            }
                        })
                    } else {
                        Toast.makeText(this, "获取文件失败", Toast.LENGTH_SHORT).show()
                    }
                }
            }
        }
    }
}