package org.brynhild.graduation.activity.admin


import android.Manifest
import android.app.AlertDialog
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.os.Build
import android.os.Bundle
import android.os.Handler
import android.util.Log
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuItem
import android.widget.Toast
import androidx.core.view.GravityCompat
import androidx.recyclerview.widget.GridLayoutManager
import com.bumptech.glide.Glide
import com.example.devicemanager.view.ConfirmPopupWindow
import com.google.gson.Gson
import org.brynhild.graduation.R
import org.brynhild.graduation.activity.common.AccountInfoActivity
import org.brynhild.graduation.activity.common.BaseActivity
import org.brynhild.graduation.activity.common.LoginActivity
import org.brynhild.graduation.common.callback.DefaultCallback
import org.brynhild.graduation.common.callback.RecyclerViewCallback
import org.brynhild.graduation.common.config.LoginConfiguration
import org.brynhild.graduation.common.config.MyApplication
import org.brynhild.graduation.common.constant.FileStorageConstant
import org.brynhild.graduation.common.transfer.Result
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.device.*
import org.brynhild.graduation.common.transfer.vo.timetable.QueryCourseRequest
import org.brynhild.graduation.common.transfer.vo.user.*
import org.brynhild.graduation.common.utils.JsonUtils
import org.brynhild.graduation.common.utils.MultipleSelectHandler
import org.brynhild.graduation.databinding.*
import org.brynhild.graduation.network.adapter.*
import org.brynhild.graduation.network.entiity.*
import org.brynhild.graduation.service.http.AccountService
import org.brynhild.graduation.service.http.AdminService
import org.brynhild.graduation.service.mqtt.MqttListenService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class AdminMainPageActivity : BaseActivity() {

    //
    private var currentPage = 0
    private val loadUserList = 0
    private val refreshUserList = 1
    private val loadAcademyList = 2
    private val refreshAcademyList = 3
    private val loadClassList = 4
    private val refreshClassList = 5
    private val loadMajorList = 6
    private val refreshMajorList = 7
    private val loadAreaList = 8;
    private val refreshAreaList = 9
    private val loadDeviceList = 10
    private val refreshDeviceList = 11
    private val loadCourseList = 12
    private val refreshCourseList = 13

    private val twoColumnLayout = GridLayoutManager(this, 2)
    private val singleColumnLayout = GridLayoutManager(this, 1)

    private var lastBack = 0L
    private lateinit var activityMainPageBinding: ActivityMainPageBinding
    private lateinit var navHeaderBinding: NavHeaderBinding

    private fun getPermission() {
        if (Build.VERSION.SDK_INT >= 23) {
            val REQUEST_CODE_CONTACT = 101
            val permissions = arrayOf<String>(Manifest.permission.WRITE_EXTERNAL_STORAGE)
            //验证是否许可权限
            for (str in permissions) {
                if (this.checkSelfPermission(str) != PackageManager.PERMISSION_GRANTED) {
                    //申请权限
                    this.requestPermissions(permissions, REQUEST_CODE_CONTACT)
                }
            }
            FileStorageConstant.initStorage()
        }
    }

    private val handler: Handler = Handler {
        when (it.what) {
            loadUserList -> {
                switchToUser {}
            }
            refreshUserList -> {
                switchToUser {
                    activityMainPageBinding.swipeRefresh.isRefreshing = false
                    Toast.makeText(this, "数据更新完成", Toast.LENGTH_SHORT).show()
                }
            }
            loadAcademyList -> {
                switchToAcademy {}
            }
            refreshAcademyList -> {
                switchToAcademy {
                    activityMainPageBinding.swipeRefresh.isRefreshing = false
                    Toast.makeText(this, "数据更新完成", Toast.LENGTH_SHORT).show()
                }
            }
            loadMajorList -> {
                switchToMajor {}
            }
            refreshMajorList -> {
                switchToMajor {
                    activityMainPageBinding.swipeRefresh.isRefreshing = false
                    Toast.makeText(this, "数据更新完成", Toast.LENGTH_SHORT).show()
                }
            }
            loadCourseList -> {
                switchToCourse {}
            }
            refreshCourseList -> {
                switchToCourse {
                    activityMainPageBinding.swipeRefresh.isRefreshing = false
                    Toast.makeText(this, "数据更新完成", Toast.LENGTH_SHORT).show()
                }
            }
            loadClassList -> {
                switchToClass {}
            }
            refreshClassList -> {
                switchToClass {
                    activityMainPageBinding.swipeRefresh.isRefreshing = false
                    Toast.makeText(this, "数据更新完成", Toast.LENGTH_SHORT).show()
                }
            }
            loadAreaList -> {
                switchToArea {}
            }
            refreshAreaList -> {
                switchToArea {
                    activityMainPageBinding.swipeRefresh.isRefreshing = false
                    Toast.makeText(this, "数据更新完成", Toast.LENGTH_SHORT).show()
                }
            }
            loadDeviceList -> {
                switchToDevice {}
            }
            refreshDeviceList -> {
                switchToDevice {
                    activityMainPageBinding.swipeRefresh.isRefreshing = false
                    Toast.makeText(this, "数据更新完成", Toast.LENGTH_SHORT).show()
                }
            }
        }
        false
    }

    private fun switchToUser(callback: DefaultCallback) {
        activityMainPageBinding.recyclerView.layoutManager = twoColumnLayout
        activityMainPageBinding.toolBar.menu.clear()
        menuInflater.inflate(R.menu.admin_user_nav_bar, activityMainPageBinding.toolBar.menu)
        val service = ServiceCreator.create(AdminService::class.java)
        val data = QueryUserRequest(null, null, null, null, null, null, null, null)
        Log.d("Admin", "查询用户列表：${data}")
        service.queryUser(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "AddDeviceActivity",
                            "flag is ${result.flag},message is ${result.message},data is ${result.data}"
                        )
                        val userList = ArrayList<User>()
                        val gson = Gson()
                        for (item in result.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val user = JsonUtils.fromJson<User>(json)
                            if (user != null) {
                                userList.add(user)
                            }
                        }
                        val adapter = UserAdapter(this@AdminMainPageActivity, userList)

                        activityMainPageBinding.recyclerView.adapter =
                            MultipleSelectHandler.createHandler(
                                this@AdminMainPageActivity,
                                adapter,
                                object : RecyclerViewCallback {
                                    override fun execute(selected: List<Int>) {
                                        val builder = StringBuilder()
                                        println(selected)
                                        var first = true

                                        val rmList = ArrayList<Long>()
                                        for (index in selected) {
                                            rmList.add(userList[index].id)
                                            if (!first) {
                                                builder.append("、")
                                            } else {
                                                first = false
                                            }
                                            builder.append(userList[index].name)
                                        }
                                        ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@AdminMainPageActivity)
                                            .setContent("以下用户将被删除：${builder}")
                                            .setCancelText("取消删除")
                                            .setEnsureText("确定删除")
                                            .setEnsureListener {

                                                val service =
                                                    ServiceCreator.create(AdminService::class.java)
                                                val rmData = AdminDeleteUser(rmList)
                                                println("send data with:${Gson().toJson(rmData)}")
                                                service.deleteUser(
                                                    LoginConfiguration.userInfo!!.token,
                                                    rmData
                                                ).enqueue(object : Callback<Result> {
                                                    override fun onResponse(
                                                        call: Call<Result>,
                                                        response: Response<Result>
                                                    ) {
                                                        ResponseHandler.handle(response, {
                                                            handler.sendEmptyMessage(refreshUserList)
                                                        }, {
                                                            ResponseHandler.showMessage(it)
                                                        })
                                                    }

                                                    override fun onFailure(
                                                        call: Call<Result>,
                                                        t: Throwable
                                                    ) {
                                                        t.printStackTrace()
                                                    }
                                                })

                                            }
                                            .build()
                                            .show()
                                    }
                                })
                        supportActionBar?.let { bar ->
                            bar.title = "用户列表"
                        }
                        callback.execute()
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
    }

    private fun switchToAcademy(callback: DefaultCallback) {
        activityMainPageBinding.recyclerView.layoutManager = twoColumnLayout
        activityMainPageBinding.toolBar.menu.clear()
        menuInflater.inflate(R.menu.admin_academy_nav_bar, activityMainPageBinding.toolBar.menu)
        val service = ServiceCreator.create(AdminService::class.java)
        val data = QueryAcademyRequest(null, null, null)
        Log.d("Admin", "查询学院列表：${data}")
        service.queryAcademy(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "AddDeviceActivity",
                            "flag is ${result.flag},message is ${result.message},data is ${result.data}"
                        )
                        val academyArrayList = ArrayList<Academy>()
                        val gson = Gson()
                        for (item in result.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val academy = JsonUtils.fromJson<Academy>(json)
                            if (academy != null) {
                                academyArrayList.add(academy)
                            }
                        }
                        val adapter = AcademyAdapter(this@AdminMainPageActivity, academyArrayList)

                        activityMainPageBinding.recyclerView.adapter =
                            MultipleSelectHandler.createHandler(
                                this@AdminMainPageActivity,
                                adapter,
                                object : RecyclerViewCallback {
                                    override fun execute(selected: List<Int>) {
                                        val builder = StringBuilder()
                                        println(selected)
                                        var first = true

                                        val rmList = ArrayList<Long>()
                                        for (index in selected) {
                                            rmList.add(academyArrayList[index].id)
                                            if (!first) {
                                                builder.append("、")
                                            } else {
                                                first = false
                                            }
                                            builder.append(academyArrayList[index].name)
                                        }
                                        ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@AdminMainPageActivity)
                                            .setContent("以下学院将被删除：${builder}")
                                            .setCancelText("取消删除")
                                            .setEnsureText("确定删除")
                                            .setEnsureListener {

                                                val service =
                                                    ServiceCreator.create(AdminService::class.java)
                                                val rmData = AdminDeleteAcademy(rmList)
                                                println("send data with:${Gson().toJson(rmData)}")
                                                service.deleteAcademy(
                                                    LoginConfiguration.userInfo!!.token,
                                                    rmData
                                                ).enqueue(object : Callback<Result> {
                                                    override fun onResponse(
                                                        call: Call<Result>,
                                                        response: Response<Result>
                                                    ) {
                                                        ResponseHandler.handle(response, {
                                                            handler.sendEmptyMessage(
                                                                refreshAcademyList
                                                            )
                                                        }, {
                                                            ResponseHandler.showMessage(it)
                                                        })
                                                    }

                                                    override fun onFailure(
                                                        call: Call<Result>,
                                                        t: Throwable
                                                    ) {
                                                        t.printStackTrace()
                                                    }
                                                })

                                            }
                                            .build()
                                            .show()
                                    }
                                })
                        supportActionBar?.let { bar ->
                            bar.title = "学院列表"
                        }
                        callback.execute()
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
    }

    private fun switchToMajor(callback: DefaultCallback) {
        activityMainPageBinding.recyclerView.layoutManager = singleColumnLayout
        activityMainPageBinding.toolBar.menu.clear()
        menuInflater.inflate(R.menu.admin_major_nav_bar, activityMainPageBinding.toolBar.menu)
        val service = ServiceCreator.create(AdminService::class.java)
        val data = QueryMajorRequest(null, null, null)
        Log.d("Admin", "查询专业列表：${data}")
        service.queryMajor(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "AddDeviceActivity",
                            "flag is ${result.flag},message is ${result.message},data is ${result.data}"
                        )
                        val majorArrayList = ArrayList<Major>()
                        val gson = Gson()
                        for (item in result.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val major = JsonUtils.fromJson<Major>(json)
                            if (major != null) {
                                majorArrayList.add(major)
                            }
                        }
                        val adapter = MajorAdapter(this@AdminMainPageActivity, majorArrayList)

                        activityMainPageBinding.recyclerView.adapter =
                            MultipleSelectHandler.createHandler(
                                this@AdminMainPageActivity,
                                adapter,
                                object : RecyclerViewCallback {
                                    override fun execute(selected: List<Int>) {
                                        val builder = StringBuilder()
                                        println(selected)
                                        var first = true

                                        val rmList = ArrayList<Long>()
                                        for (index in selected) {
                                            rmList.add(majorArrayList[index].id)
                                            if (!first) {
                                                builder.append("、")
                                            } else {
                                                first = false
                                            }
                                            builder.append(majorArrayList[index].name)
                                        }
                                        ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@AdminMainPageActivity)
                                            .setContent("以下专业将被删除：${builder}")
                                            .setCancelText("取消删除")
                                            .setEnsureText("确定删除")
                                            .setEnsureListener {

                                                val service =
                                                    ServiceCreator.create(AdminService::class.java)
                                                val rmData = AdminDeleteMajor(rmList)
                                                println("send data with:${Gson().toJson(rmData)}")
                                                service.deleteMajor(
                                                    LoginConfiguration.userInfo!!.token,
                                                    rmData
                                                ).enqueue(object : Callback<Result> {
                                                    override fun onResponse(
                                                        call: Call<Result>,
                                                        response: Response<Result>
                                                    ) {
                                                        ResponseHandler.handle(response, {
                                                            handler.sendEmptyMessage(
                                                                refreshMajorList
                                                            )
                                                        }, {
                                                            ResponseHandler.showMessage(it)
                                                        })
                                                    }

                                                    override fun onFailure(
                                                        call: Call<Result>,
                                                        t: Throwable
                                                    ) {
                                                        t.printStackTrace()
                                                    }
                                                })

                                            }
                                            .build()
                                            .show()
                                    }
                                })
                        supportActionBar?.let { bar ->
                            bar.title = "专业列表"
                        }
                        callback.execute()
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
    }

    private fun switchToClass(callback: DefaultCallback) {
        activityMainPageBinding.recyclerView.layoutManager = singleColumnLayout
        activityMainPageBinding.toolBar.menu.clear()
        menuInflater.inflate(R.menu.admin_class_nav_bar, activityMainPageBinding.toolBar.menu)
        val service = ServiceCreator.create(AdminService::class.java)
        val data = QueryClassRequest(null, null, null, null)
        Log.d("Admin", "查询班级列表：${data}")
        service.queryClass(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "AddDeviceActivity",
                            "flag is ${result.flag},message is ${result.message},data is ${result.data}"
                        )
                        val classEntityArrayList = ArrayList<ClassEntity>()
                        val gson = Gson()
                        for (item in result.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val classEntity = JsonUtils.fromJson<ClassEntity>(json)
                            if (classEntity != null) {
                                classEntityArrayList.add(classEntity)
                            }
                        }
                        val adapter =
                            ClassEntityAdapter(this@AdminMainPageActivity, classEntityArrayList)

                        activityMainPageBinding.recyclerView.adapter =
                            MultipleSelectHandler.createHandler(
                                this@AdminMainPageActivity,
                                adapter,
                                object : RecyclerViewCallback {
                                    override fun execute(selected: List<Int>) {
                                        val builder = StringBuilder()
                                        println(selected)
                                        var first = true

                                        val rmList = ArrayList<Long>()
                                        for (index in selected) {
                                            rmList.add(classEntityArrayList[index].id)
                                            if (!first) {
                                                builder.append("、")
                                            } else {
                                                first = false
                                            }
                                            builder.append(classEntityArrayList[index].name)
                                        }
                                        ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@AdminMainPageActivity)
                                            .setContent("以下班级将被删除：${builder}")
                                            .setCancelText("取消删除")
                                            .setEnsureText("确定删除")
                                            .setEnsureListener {

                                                val service =
                                                    ServiceCreator.create(AdminService::class.java)
                                                val rmData = AdminDeleteClass(rmList)
                                                println("send data with:${Gson().toJson(rmData)}")
                                                service.deleteClass(
                                                    LoginConfiguration.userInfo!!.token,
                                                    rmData
                                                ).enqueue(object : Callback<Result> {
                                                    override fun onResponse(
                                                        call: Call<Result>,
                                                        response: Response<Result>
                                                    ) {
                                                        ResponseHandler.handle(response, {
                                                            handler.sendEmptyMessage(
                                                                refreshClassList
                                                            )
                                                        }, {
                                                            ResponseHandler.showMessage(it)
                                                        })
                                                    }

                                                    override fun onFailure(
                                                        call: Call<Result>,
                                                        t: Throwable
                                                    ) {
                                                        t.printStackTrace()
                                                    }
                                                })

                                            }
                                            .build()
                                            .show()
                                    }
                                })
                        supportActionBar?.let { bar ->
                            bar.title = "班级列表"
                        }
                        callback.execute()
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
    }

    private fun switchToArea(callback: DefaultCallback) {
        activityMainPageBinding.recyclerView.layoutManager = singleColumnLayout
        activityMainPageBinding.toolBar.menu.clear()
        menuInflater.inflate(R.menu.admin_area_nav_bar, activityMainPageBinding.toolBar.menu)
        val service = ServiceCreator.create(AdminService::class.java)
        val data = QueryAreaRequest(null, null)
        Log.d("Admin", "查询区域列表：${data}")
        service.queryArea(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "AddDeviceActivity",
                            "flag is ${result.flag},message is ${result.message},data is ${result.data}"
                        )
                        val areaArrayList = ArrayList<Area>()
                        val gson = Gson()
                        for (item in result.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val area = JsonUtils.fromJson<Area>(json)
                            if (area != null) {
                                areaArrayList.add(area)
                            }
                        }
                        val adapter = AreaAdapter(this@AdminMainPageActivity, areaArrayList)

                        activityMainPageBinding.recyclerView.adapter =
                            MultipleSelectHandler.createHandler(
                                this@AdminMainPageActivity,
                                adapter,
                                object : RecyclerViewCallback {
                                    override fun execute(selected: List<Int>) {
                                        val builder = StringBuilder()
                                        println(selected)
                                        var first = true

                                        val rmList = ArrayList<Long>()
                                        for (index in selected) {
                                            rmList.add(areaArrayList[index].id)
                                            if (!first) {
                                                builder.append("、")
                                            } else {
                                                first = false
                                            }
                                            builder.append(areaArrayList[index].name)
                                        }
                                        ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@AdminMainPageActivity)
                                            .setContent("以下区域将被删除：${builder}")
                                            .setCancelText("取消删除")
                                            .setEnsureText("确定删除")
                                            .setEnsureListener {

                                                val service =
                                                    ServiceCreator.create(AdminService::class.java)
                                                val rmData = AdminDeleteArea(rmList)
                                                println("send data with:${Gson().toJson(rmData)}")
                                                service.deleteArea(
                                                    LoginConfiguration.userInfo!!.token,
                                                    rmData
                                                ).enqueue(object : Callback<Result> {
                                                    override fun onResponse(
                                                        call: Call<Result>,
                                                        response: Response<Result>
                                                    ) {
                                                        ResponseHandler.handle(response, {
                                                            handler.sendEmptyMessage(refreshAreaList)
                                                        }, {
                                                            ResponseHandler.showMessage(it)
                                                        })
                                                    }

                                                    override fun onFailure(
                                                        call: Call<Result>,
                                                        t: Throwable
                                                    ) {
                                                        t.printStackTrace()
                                                    }
                                                })

                                            }
                                            .build()
                                            .show()
                                    }
                                })
                        supportActionBar?.let { bar ->
                            bar.title = "教室列表"
                        }
                        callback.execute()
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
    }

    private fun switchToDevice(callback: DefaultCallback) {
        activityMainPageBinding.recyclerView.layoutManager = singleColumnLayout
        activityMainPageBinding.toolBar.menu.clear()
        val service = ServiceCreator.create(AdminService::class.java)
        val data = QueryDeviceRequest(null, null, null, null, null, null, null)
        Log.d("Admin", "查询设备列表：${data}")
        service.queryDevice(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "AddDeviceActivity",
                            "flag is ${result.flag},message is ${result.message},data is ${result.data}"
                        )
                        val deviceArrayList = ArrayList<Device>()
                        val gson = Gson()
                        for (item in result.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val device = JsonUtils.fromJson<Device>(json)
                            if (device != null) {
                                deviceArrayList.add(device)
                            }
                        }
                        val adapter = DeviceAdapter(this@AdminMainPageActivity, deviceArrayList)

                        activityMainPageBinding.recyclerView.adapter =
                            MultipleSelectHandler.createHandler(
                                this@AdminMainPageActivity,
                                adapter,
                                object : RecyclerViewCallback {
                                    override fun execute(selected: List<Int>) {
                                        val builder = StringBuilder()
                                        println(selected)
                                        var first = true

                                        val rmList = ArrayList<Long>()
                                        for (index in selected) {
                                            rmList.add(deviceArrayList[index].id)
                                            if (!first) {
                                                builder.append("、")
                                            } else {
                                                first = false
                                            }
                                            builder.append(deviceArrayList[index].name)
                                        }
                                        ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@AdminMainPageActivity)
                                            .setContent("以下设备将被删除：${builder}")
                                            .setCancelText("取消删除")
                                            .setEnsureText("确定删除")
                                            .setEnsureListener {

                                                val service =
                                                    ServiceCreator.create(AdminService::class.java)
                                                val rmData = AdminDeleteDevice(rmList)
                                                println("send data with:${Gson().toJson(rmData)}")
                                                service.deleteDevice(
                                                    LoginConfiguration.userInfo!!.token,
                                                    rmData
                                                ).enqueue(object : Callback<Result> {
                                                    override fun onResponse(
                                                        call: Call<Result>,
                                                        response: Response<Result>
                                                    ) {
                                                        ResponseHandler.handle(response, {
                                                            handler.sendEmptyMessage(
                                                                refreshDeviceList
                                                            )
                                                        }, {
                                                            ResponseHandler.showMessage(it)
                                                        })
                                                    }

                                                    override fun onFailure(
                                                        call: Call<Result>,
                                                        t: Throwable
                                                    ) {
                                                        t.printStackTrace()
                                                    }
                                                })

                                            }
                                            .build()
                                            .show()
                                    }
                                })
                        supportActionBar?.let { bar ->
                            bar.title = "设备列表"
                        }
                        callback.execute()
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
    }

    private fun switchToCourse(callback: DefaultCallback) {
        activityMainPageBinding.recyclerView.layoutManager = singleColumnLayout
        activityMainPageBinding.toolBar.menu.clear()
        val service = ServiceCreator.create(AdminService::class.java)
        val data = QueryCourseRequest(null, null, null, null)
        Log.d("Admin", "查询课程列表：${data}")
        service.queryCourse(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "AddDeviceActivity",
                            "flag is ${result.flag},message is ${result.message},data is ${result.data}"
                        )
                        val courseArrayList = ArrayList<Course>()
                        val gson = Gson()
                        for (item in result.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val course = JsonUtils.fromJson<Course>(json)
                            if (course != null) {
                                courseArrayList.add(course)
                            }
                        }
                        val adapter = CourseAdapter(this@AdminMainPageActivity, courseArrayList)

                        activityMainPageBinding.recyclerView.adapter =
                            MultipleSelectHandler.createHandler(
                                this@AdminMainPageActivity,
                                adapter,
                                object : RecyclerViewCallback {
                                    override fun execute(selected: List<Int>) {
                                        val builder = StringBuilder()
                                        println(selected)
                                        var first = true

                                        val rmList = ArrayList<Long>()
                                        for (index in selected) {
                                            rmList.add(courseArrayList[index].id)
                                            if (!first) {
                                                builder.append("、")
                                            } else {
                                                first = false
                                            }
                                            builder.append(courseArrayList[index].name)
                                        }
                                        ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@AdminMainPageActivity)
                                            .setContent("以下设备将被删除：${builder}")
                                            .setCancelText("取消删除")
                                            .setEnsureText("确定删除")
                                            .setEnsureListener {

                                                val service =
                                                    ServiceCreator.create(AdminService::class.java)
                                                val rmData = AdminDeleteDevice(rmList)
                                                println("send data with:${Gson().toJson(rmData)}")
                                                service.deleteDevice(
                                                    LoginConfiguration.userInfo!!.token,
                                                    rmData
                                                ).enqueue(object : Callback<Result> {
                                                    override fun onResponse(
                                                        call: Call<Result>,
                                                        response: Response<Result>
                                                    ) {
                                                        ResponseHandler.handle(response, {
                                                            handler.sendEmptyMessage(
                                                                refreshDeviceList
                                                            )
                                                        }, {
                                                            ResponseHandler.showMessage(it)
                                                        })
                                                    }

                                                    override fun onFailure(
                                                        call: Call<Result>,
                                                        t: Throwable
                                                    ) {
                                                        t.printStackTrace()
                                                    }
                                                })

                                            }
                                            .build()
                                            .show()
                                    }
                                })
                        supportActionBar?.let { bar ->
                            bar.title = "课程列表"
                        }
                        callback.execute()
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        activityMainPageBinding = ActivityMainPageBinding.inflate(LayoutInflater.from(this))
        setContentView(activityMainPageBinding.root)

        LoginConfiguration.loadTimeTableConfig {

        }

        setSupportActionBar(activityMainPageBinding.toolBar)
        supportActionBar?.let {
            it.setDisplayHomeAsUpEnabled(true)
            it.setHomeAsUpIndicator(R.drawable.ic_menu)
            it.title = "User"
        }

        getPermission()

        activityMainPageBinding.recyclerView.layoutManager = twoColumnLayout
        activityMainPageBinding.navView.setCheckedItem(R.id.navUser)
        currentPage = R.id.navUser

        activityMainPageBinding.navView.setNavigationItemSelectedListener { it ->
            when (it.itemId) {
                R.id.navUser -> {
                    currentPage = R.id.navUser
                    handler.sendEmptyMessage(loadUserList)
                }

                R.id.navAcademy -> {
                    currentPage = R.id.navAcademy
                    switchToAcademy {}
                }

                R.id.navMajor -> {
                    currentPage = R.id.navMajor
                    switchToMajor {}
                }

                R.id.navCourse -> {
                    currentPage = R.id.navCourse
                    switchToCourse {}
                }

                R.id.navClass -> {
                    currentPage = R.id.navClass
                    switchToClass {}
                }

                R.id.navArea -> {
                    currentPage = R.id.navArea
                    switchToArea {}
                }

                R.id.navDevice -> {
                    currentPage = R.id.navDevice
                    switchToDevice {}
                }

                R.id.navExit -> {
                    LoginConfiguration.login_password = ""
                    saveLoginInfo()
                    val i = Intent(MyApplication.context, LoginActivity::class.java)
                    startActivity(i)
                    finish()
                }
            }
            activityMainPageBinding.drawerLayout.closeDrawers()
            true
        }
        activityMainPageBinding.swipeRefresh.setOnRefreshListener {
            Thread {
                when (currentPage) {
                    R.id.navUser -> {
                        handler.sendEmptyMessage(refreshUserList)
                    }

                    R.id.navAcademy -> {
                        handler.sendEmptyMessage(refreshAcademyList)
                    }

                    R.id.navMajor -> {
                        handler.sendEmptyMessage(refreshMajorList)
                    }

                    R.id.navCourse -> {
                        handler.sendEmptyMessage(refreshCourseList)
                    }

                    R.id.navClass -> {
                        handler.sendEmptyMessage(refreshClassList)
                    }

                    R.id.navArea -> {
                        handler.sendEmptyMessage(refreshAreaList)
                    }

                    R.id.navDevice -> {
                        handler.sendEmptyMessage(refreshDeviceList)
                    }

                    R.id.navExit -> {
                        LoginConfiguration.login_password = ""
                        saveLoginInfo()
                        val serviceIntent =
                            Intent(MyApplication.context, MqttListenService::class.java)
                        stopService(serviceIntent)
                        val i = Intent(MyApplication.context, LoginActivity::class.java)
                        startActivity(i)
                        finish()
                    }
                }

            }.start()
        }

        val headerView = activityMainPageBinding.navView.inflateHeaderView(R.layout.nav_header)
        navHeaderBinding = NavHeaderBinding.bind(headerView)
        navHeaderBinding.navProfile.setOnClickListener {
            val intent = Intent(this, AccountInfoActivity::class.java)
            intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
            this.startActivity(intent)
        }

        switchToUser {}
    }

    private fun loadUserInfo() {
        Thread {
            val service = ServiceCreator.create(AccountService::class.java)
            service.accountInfo(LoginConfiguration.userInfo!!.token)
                .enqueue(object : Callback<Result> {
                    override fun onResponse(call: Call<Result>, response: Response<Result>) {
                        ResponseHandler.handle(response, {
                            val json = Gson().toJson(it.data)
                            val user = JsonUtils.fromJson<User>(json)
                            if (user != null) {
                                navHeaderBinding.userText.text = user.name
                                navHeaderBinding.mailText.text = user.email
                                Glide.with(this@AdminMainPageActivity)
                                    .load(user.avatar ?: R.drawable.nav_icon_default)
                                    .into(navHeaderBinding.iconImage)
                            }
                        }, {
                            Toast.makeText(MyApplication.context, "请求用户基本信息失败", Toast.LENGTH_SHORT)
                                .show()
                        })
                    }

                    override fun onFailure(call: Call<Result>, t: Throwable) {
                        Toast.makeText(MyApplication.context, "请求用户基本信息失败", Toast.LENGTH_SHORT)
                            .show()
                    }
                })
        }.start()
    }

    override fun onResume() {
        super.onResume()
        loadUserInfo()
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.admin_user_nav_bar, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            android.R.id.home -> activityMainPageBinding.drawerLayout.openDrawer(GravityCompat.START)
            R.id.admin_user_add -> {
                val dialogLayout = AdminCreateUserDialogBinding.inflate(LayoutInflater.from(this))

                AlertDialog.Builder(this)
                    .setTitle("添加学生")
                    .setView(dialogLayout.root)
                    .setPositiveButton("确认添加") { _, _ ->
                        val service = ServiceCreator.create(AdminService::class.java)
                        val data = AdminAddUser(
                            dialogLayout.username.text.toString(),
                            dialogLayout.password.text.toString(),
                            dialogLayout.name.text.toString(),
                            dialogLayout.sex.text.toString(),
                            dialogLayout.departmentId.text.toString().toLongOrNull(),
                            dialogLayout.no.text.toString()
                        )
                        service.addUser(LoginConfiguration.userInfo!!.token, data)
                            .enqueue(object : Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        handler.sendEmptyMessage(refreshUserList)
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .setNegativeButton("取消添加") { _, _ ->

                    }
                    .show()
            }

            R.id.admin_academy_add -> {
                val dialogLayout =
                    AdminCreateAcademyDialogBinding.inflate(LayoutInflater.from(this))

                AlertDialog.Builder(this)
                    .setTitle("添加学院")
                    .setView(dialogLayout.root)
                    .setPositiveButton("确认添加") { _, _ ->
                        val service = ServiceCreator.create(AdminService::class.java)
                        val data = AdminAddAcademy(
                            dialogLayout.no.text.toString(),
                            dialogLayout.name.text.toString(),
                            dialogLayout.liable.text.toString().toLongOrNull()
                        )
                        service.addAcademy(LoginConfiguration.userInfo!!.token, data)
                            .enqueue(object : Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        handler.sendEmptyMessage(refreshAcademyList)
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .setNegativeButton("取消添加") { _, _ ->

                    }
                    .show()
            }

            R.id.admin_major_add -> {

            }

            R.id.admin_class_add -> {

            }

            R.id.admin_area_add -> {
                val dialogLayout = AdminCreateAreaDialogBinding.inflate(LayoutInflater.from(this))

                AlertDialog.Builder(this)
                    .setTitle("添加区域")
                    .setView(dialogLayout.root)
                    .setPositiveButton("确认添加") { _, _ ->
                        val service = ServiceCreator.create(AdminService::class.java)
                        val data = AdminAddArea(
                            dialogLayout.liable.text.toString().toLongOrNull(),
                            dialogLayout.name.text.toString(),
                            dialogLayout.note.text.toString()
                        )
                        service.addArea(LoginConfiguration.userInfo!!.token, data)
                            .enqueue(object : Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        handler.sendEmptyMessage(refreshAreaList)
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .setNegativeButton("取消添加") { _, _ ->

                    }
                    .show()
            }

        }
        return true
    }

    override fun onBackPressed() {
        if (lastBack == 0L || System.currentTimeMillis() - lastBack > 2000) {
            Toast.makeText(this, "再按一次退出程序", Toast.LENGTH_SHORT).show()
            lastBack = System.currentTimeMillis()
        } else {
            val serviceIntent = Intent(MyApplication.context, MqttListenService::class.java)
            stopService(serviceIntent)
            finish()
        }
    }

    private fun saveLoginInfo() {
        val accountName = LoginConfiguration.login_username
        val accountPassword = LoginConfiguration.login_password
        val sp = getSharedPreferences("account", Context.MODE_PRIVATE)
        with(sp.edit()) {
            putString("accountName", accountName)
            putString("accountPassword", accountPassword)
            apply()
        }
        Log.d("LoginActivity", "save info:name:$accountName,password:$accountPassword")
    }
}