package org.brynhild.graduation.activity.admin

import android.app.AlertDialog
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.util.Log
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuItem
import androidx.recyclerview.widget.GridLayoutManager
import com.example.devicemanager.view.ConfirmPopupWindow
import com.google.gson.Gson
import org.brynhild.graduation.R
import org.brynhild.graduation.activity.common.BaseActivity
import org.brynhild.graduation.activity.common.StatisticActivity
import org.brynhild.graduation.activity.common.TimetableActivity
import org.brynhild.graduation.common.callback.RecyclerViewCallback
import org.brynhild.graduation.common.config.LoginConfiguration
import org.brynhild.graduation.common.constant.StatisticConstant
import org.brynhild.graduation.common.transfer.Result
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.device.AdminClearFingerPrintDataAtDevice
import org.brynhild.graduation.common.transfer.vo.device.AdminGetClassFingerPrintDataIdList
import org.brynhild.graduation.common.transfer.vo.device.AdminPushFingerPrintDataToDevice
import org.brynhild.graduation.common.transfer.vo.device.EnrollFingerPrint
import org.brynhild.graduation.common.transfer.vo.user.*
import org.brynhild.graduation.common.utils.JsonUtils
import org.brynhild.graduation.common.utils.MultipleSelectHandler
import org.brynhild.graduation.databinding.*
import org.brynhild.graduation.network.adapter.UserAdapter
import org.brynhild.graduation.network.entiity.ClassEntity
import org.brynhild.graduation.network.entiity.User
import org.brynhild.graduation.service.http.AdminService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response


class ClassDetailInfoActivity : BaseActivity() {
    private lateinit var binding: ActivityClassDetailInfoBinding
    private lateinit var classEntity: ClassEntity
    private val refreshStudentList = 0
    private var continueEnroll = true
    private var enrollDeviceMac = ""
    private lateinit var userList: ArrayList<User>

    val handler: Handler = Handler {
        when (it.what) {
            refreshStudentList -> {
                loadStudentList()
            }
        }
        false
    }

    private fun loadStudentList() {
        val service = ServiceCreator.create(AdminService::class.java)
        val data = QueryUserRequest(null, null, null, null, null, null, null, classEntity.id)
        Log.d("Admin", "查询班级列表：${data}")
        service.queryUser(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "AddDeviceActivity",
                            "flag is ${result.flag},message is ${result.message},data is ${result.data}"
                        )
                        val userArrayList = ArrayList<User>()
                        val gson = Gson()
                        for (item in result.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val user = JsonUtils.fromJson<User>(json)
                            if (user != null) {
                                userArrayList.add(user)
                            }
                        }
                        userList = userArrayList
                        val adapter = UserAdapter(this@ClassDetailInfoActivity, userArrayList)

                        binding.recyclerView.adapter = MultipleSelectHandler.createHandler(
                            this@ClassDetailInfoActivity,
                            adapter,
                            object : RecyclerViewCallback {
                                override fun execute(selected: List<Int>) {
                                    val builder = StringBuilder()
                                    println(selected)
                                    var first = true

                                    val rmList = ArrayList<Long>()
                                    for (index in selected) {
                                        rmList.add(userArrayList[index].id)
                                        if (!first) {
                                            builder.append("、")
                                        } else {
                                            first = false
                                        }
                                        builder.append(userArrayList[index].name)
                                    }
                                    ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@ClassDetailInfoActivity)
                                        .setContent("以下用户将被删除：${builder}")
                                        .setCancelText("取消删除")
                                        .setEnsureText("确定删除")
                                        .setEnsureListener {

                                            val service =
                                                ServiceCreator.create(AdminService::class.java)
                                            val rmData = AdminDeleteUser(rmList)
                                            println("send data with:${Gson().toJson(rmData)}")
                                            service.deleteUser(
                                                LoginConfiguration.userInfo!!.token,
                                                rmData
                                            ).enqueue(object : Callback<Result> {
                                                override fun onResponse(
                                                    call: Call<Result>,
                                                    response: Response<Result>
                                                ) {
                                                    ResponseHandler.handle(response, {
                                                        handler.sendEmptyMessage(refreshStudentList)
                                                    }, {
                                                        ResponseHandler.showMessage(it)
                                                    })
                                                }

                                                override fun onFailure(
                                                    call: Call<Result>,
                                                    t: Throwable
                                                ) {
                                                    t.printStackTrace()
                                                }
                                            })

                                        }
                                        .build()
                                        .show()
                                }
                            })
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityClassDetailInfoBinding.inflate(LayoutInflater.from(this))
        setContentView(binding.root)
        setSupportActionBar(binding.toolBar)
        val layoutManager = GridLayoutManager(this, 1)
        binding.recyclerView.layoutManager = layoutManager

        val gson = Gson()
        classEntity = gson.fromJson(intent.getStringExtra("class"), ClassEntity::class.java)

        setTableData(classEntity)

        bindClick()

        loadStudentList()
    }

    private fun bindClick() {
        binding.backButton.setOnClickListener {
            finish()
        }

        binding.adminClassStatistic.setOnClickListener {
            val intent = Intent(this, StatisticActivity::class.java)
            intent.putExtra(StatisticConstant.ACTION, StatisticConstant.CLASS_ACTION)
            intent.putExtra(StatisticConstant.ID_ENUM, classEntity.id)
            startActivity(intent)
        }

        binding.adminClassBatchEnroll.setOnClickListener {
            val dialogLayout =
                AdminClassBatchEnrollDeviceMacDialogBinding.inflate(LayoutInflater.from(this))
            AlertDialog.Builder(this)
                .setTitle("输入设备MAC以继续")
                .setView(dialogLayout.root)
                .setPositiveButton("开始录入") { _, _ ->
                    continueEnroll = true
                    enrollDeviceMac = dialogLayout.mac.text.toString()
                    for (user: User in userList) {
                        if (!continueEnroll) {
                            break
                        }

                        ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@ClassDetailInfoActivity)
                            .setContent("当前录入用户：${user.name}")
                            .setCancelText("终止录入")
                            .setCancelListener {
                                continueEnroll = false
                            }
                            .setEnsureText("录入完毕")
                            .setEnsureListener {
                                val service = ServiceCreator.create(AdminService::class.java)
                                val data = EnrollFingerPrint(
                                    user.id,
                                    enrollDeviceMac
                                )
                                service.enrollFingerPrint(LoginConfiguration.userInfo!!.token, data)
                                    .enqueue(object :
                                        Callback<Result> {
                                        override fun onResponse(
                                            call: Call<Result>,
                                            response: Response<Result>
                                        ) {
                                            ResponseHandler.handle(response, {
                                                ResponseHandler.showMessage(it)
                                            }, {
                                                ResponseHandler.showMessage(it)
                                            })
                                        }

                                        override fun onFailure(call: Call<Result>, t: Throwable) {
                                            t.printStackTrace()
                                        }
                                    })
                            }
                            .build()
                            .show()
                    }
                }
                .setNegativeButton("") { _, _ ->
                    continueEnroll = false
                }
                .show()
        }

        binding.adminClassBatchPush.setOnClickListener {
            val service = ServiceCreator.create(AdminService::class.java)
            val gson = Gson()
            val dialogLayout =
                AdminClassBatchPushDeviceMacDialogBinding.inflate(LayoutInflater.from(this))
            AlertDialog.Builder(this)
                .setTitle("输入设备MAC以继续")
                .setView(dialogLayout.root)
                .setPositiveButton("开始推送") { _, _ ->
                    enrollDeviceMac = dialogLayout.mac.text.toString()
                    val getClassFingerPrintDataIdList =
                        AdminGetClassFingerPrintDataIdList(classEntity.id)
                    service.getClassFingerPrintDataIdList(
                        LoginConfiguration.userInfo!!.token,
                        getClassFingerPrintDataIdList
                    ).enqueue(object : Callback<Result> {
                        override fun onResponse(
                            call: Call<Result>,
                            response: Response<Result>
                        ) {
                            ResponseHandler.handle(response, {
                                val json = gson.toJson(it.data)
                                val idList = JsonUtils.fromJson2List<Long>(json)
                                if (idList != null) {
                                    val clearDataRequest = AdminClearFingerPrintDataAtDevice(
                                        enrollDeviceMac,
                                        classEntity.id
                                    )
                                    service.clearFingerPrintDataAtDevice(
                                        LoginConfiguration.userInfo!!.token,
                                        clearDataRequest
                                    ).enqueue(object : Callback<Result> {
                                        override fun onResponse(
                                            call: Call<Result>,
                                            response: Response<Result>
                                        ) {
                                            ResponseHandler.handle(response, {
                                                val pushData = AdminPushFingerPrintDataToDevice(
                                                    enrollDeviceMac,
                                                    idList
                                                )
                                                service.pushFingerPrintDataToDevice(
                                                    LoginConfiguration.userInfo!!.token,
                                                    pushData
                                                ).enqueue(object : Callback<Result> {
                                                    override fun onResponse(
                                                        call: Call<Result>,
                                                        response: Response<Result>
                                                    ) {
                                                        ResponseHandler.handle(response, {
                                                            ResponseHandler.showMessage(it)
                                                        }, {
                                                            ResponseHandler.showMessage(it)
                                                        })
                                                    }

                                                    override fun onFailure(
                                                        call: Call<Result>,
                                                        t: Throwable
                                                    ) {
                                                        t.printStackTrace()
                                                    }
                                                })
                                            }, {
                                                ResponseHandler.showMessage(it)
                                            })
                                        }

                                        override fun onFailure(call: Call<Result>, t: Throwable) {
                                            t.printStackTrace()
                                        }
                                    })
                                }
                            }, {
                                ResponseHandler.showMessage(it)
                            })
                        }

                        override fun onFailure(call: Call<Result>, t: Throwable) {
                            t.printStackTrace()
                        }
                    })
                }
                .setNegativeButton("") { _, _ ->
                    continueEnroll = false
                }
                .show()
        }

        binding.queryTimeTable.setOnClickListener {
            val intent = Intent(this, TimetableActivity::class.java)
            val gson = Gson()
            intent.putExtra("classEntity", gson.toJson(classEntity))
            startActivity(intent)
        }
    }

    private fun setTableData(entity: ClassEntity) {
        binding.no.text = entity.no
        binding.name.text = entity.name
        binding.title.text = entity.name
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.admin_class_detail_info_options, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            R.id.admin_class_modify -> {
                val dialogLayout = AdminModifyClassDialogBinding.inflate(LayoutInflater.from(this))
                dialogLayout.no.setText(classEntity.no)
                dialogLayout.name.setText(classEntity.name)

                AlertDialog.Builder(this)
                    .setTitle("修改信息")
                    .setView(dialogLayout.root)
                    .setPositiveButton("确认修改") { _, _ ->
                        val service = ServiceCreator.create(AdminService::class.java)
                        val data = AdminModifyClass(
                            classEntity.id,
                            dialogLayout.no.text.toString(),
                            dialogLayout.name.text.toString(),
                            null
                        )
                        service.modifyClass(LoginConfiguration.userInfo!!.token, data)
                            .enqueue(object :
                                Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        finish()
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .setNegativeButton("取消修改") { _, _ ->

                    }
                    .show()
            }

            R.id.admin_student_add -> {
                val dialogLayout = AdminCreateUserDialogBinding.inflate(LayoutInflater.from(this))

                AlertDialog.Builder(this)
                    .setTitle("添加学生")
                    .setView(dialogLayout.root)
                    .setPositiveButton("确认添加") { _, _ ->
                        val service = ServiceCreator.create(AdminService::class.java)
                        val data = AdminAddUser(
                            dialogLayout.username.text.toString(),
                            dialogLayout.password.text.toString(),
                            dialogLayout.name.text.toString(),
                            dialogLayout.sex.text.toString(),
                            classEntity.id,
                            dialogLayout.no.text.toString(),
                        )
                        service.addUser(LoginConfiguration.userInfo!!.token, data)
                            .enqueue(object : Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        handler.sendEmptyMessage(refreshStudentList)
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .setNegativeButton("取消添加") { _, _ ->

                    }
                    .show()
            }
            R.id.admin_class_delete -> {
                ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@ClassDetailInfoActivity)
                    .setContent("确定删除班级：${classEntity.name}吗")
                    .setCancelText("取消删除")
                    .setEnsureText("确定删除")
                    .setEnsureListener {
                        val service = ServiceCreator.create(AdminService::class.java)
                        val rmList = ArrayList<Long>()
                        rmList.add(classEntity.id)
                        val rmData = AdminDeleteClass(rmList)
                        service.deleteClass(LoginConfiguration.userInfo!!.token, rmData)
                            .enqueue(object :
                                Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        finish()
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .build()
                    .show()
            }
        }
        return true
    }
}