package org.brynhild.graduation.activity.admin

import android.app.AlertDialog
import android.os.Bundle
import android.os.Handler
import android.util.Log
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuItem
import androidx.recyclerview.widget.GridLayoutManager
import com.example.devicemanager.view.ConfirmPopupWindow
import com.google.gson.Gson
import org.brynhild.graduation.R
import org.brynhild.graduation.activity.common.BaseActivity
import org.brynhild.graduation.common.callback.RecyclerViewCallback
import org.brynhild.graduation.common.config.LoginConfiguration
import org.brynhild.graduation.common.transfer.Result
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.device.AdminDeleteDevice
import org.brynhild.graduation.common.transfer.vo.device.AdminModifyDevice
import org.brynhild.graduation.common.transfer.vo.device.EnrollFingerPrint
import org.brynhild.graduation.common.transfer.vo.device.QueryDeviceRequest
import org.brynhild.graduation.common.utils.JsonUtils
import org.brynhild.graduation.common.utils.MultipleSelectHandler
import org.brynhild.graduation.databinding.ActivityDeviceDetailInfoBinding
import org.brynhild.graduation.databinding.AdminEnrollFingerPrintDialogBinding
import org.brynhild.graduation.databinding.AdminModifyDeviceDialogBinding
import org.brynhild.graduation.network.adapter.DeviceAdapter
import org.brynhild.graduation.network.entiity.Device
import org.brynhild.graduation.service.http.AdminService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class DeviceDetailInfoActivity : BaseActivity() {
    private lateinit var binding: ActivityDeviceDetailInfoBinding
    private lateinit var device: Device
    private val refreshDeviceList = 0

    val handler: Handler = Handler {
        when (it.what) {
            refreshDeviceList -> {
                loadDeviceList()
            }
        }
        false
    }

    private fun loadDeviceList() {
        val service = ServiceCreator.create(AdminService::class.java)
        val data = QueryDeviceRequest(null, null, device.id, null, null, null, null)
        Log.d("Admin", "查询设备列表：${data}")
        service.queryDevice(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "AddDeviceActivity",
                            "flag is ${result.flag},message is ${result.message},data is ${result.data}"
                        )
                        val deviceArrayList = ArrayList<Device>()
                        val gson = Gson()
                        for (item in result.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val device = JsonUtils.fromJson<Device>(json)
                            if (device != null) {
                                deviceArrayList.add(device)
                            }
                        }
                        val adapter = DeviceAdapter(this@DeviceDetailInfoActivity, deviceArrayList)

                        binding.recyclerView.adapter = MultipleSelectHandler.createHandler(
                            this@DeviceDetailInfoActivity,
                            adapter,
                            object : RecyclerViewCallback {
                                override fun execute(selected: List<Int>) {
                                    val builder = StringBuilder()
                                    println(selected)
                                    var first = true

                                    val rmList = ArrayList<Long>()
                                    for (index in selected) {
                                        rmList.add(deviceArrayList[index].id)
                                        if (!first) {
                                            builder.append("、")
                                        } else {
                                            first = false
                                        }
                                        builder.append(deviceArrayList[index].name)
                                    }
                                    ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@DeviceDetailInfoActivity)
                                        .setContent("以下班级将被删除：${builder}")
                                        .setCancelText("取消删除")
                                        .setEnsureText("确定删除")
                                        .setEnsureListener {

                                            val service =
                                                ServiceCreator.create(AdminService::class.java)
                                            val rmData = AdminDeleteDevice(rmList)
                                            println("send data with:${Gson().toJson(rmData)}")
                                            service.deleteDevice(
                                                LoginConfiguration.userInfo!!.token,
                                                rmData
                                            ).enqueue(object : Callback<Result> {
                                                override fun onResponse(
                                                    call: Call<Result>,
                                                    response: Response<Result>
                                                ) {
                                                    ResponseHandler.handle(response, {
                                                        handler.sendEmptyMessage(refreshDeviceList)
                                                    }, {
                                                        ResponseHandler.showMessage(it)
                                                    })
                                                }

                                                override fun onFailure(
                                                    call: Call<Result>,
                                                    t: Throwable
                                                ) {
                                                    t.printStackTrace()
                                                }
                                            })

                                        }
                                        .build()
                                        .show()
                                }
                            })
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityDeviceDetailInfoBinding.inflate(LayoutInflater.from(this))
        setContentView(binding.root)
        setSupportActionBar(binding.toolBar)
        val layoutManager = GridLayoutManager(this, 1)
        binding.recyclerView.layoutManager = layoutManager

        val gson = Gson()
        device = gson.fromJson(
            intent.getStringExtra("device"),
            Device::class.java
        )

        setTableData(device)

        bindOnClick()

//        loadDeviceList()
    }

    private fun bindOnClick() {
        binding.backButton.setOnClickListener {
            finish()
        }
    }

    private fun setTableData(area: Device) {
        binding.owner.text = area.owner.toString()
        binding.area.text = area.area.toString()
        binding.mac.text = area.mac
        binding.lastActive.text = area.lastActive
        binding.name.text = area.name
        binding.active.text = area.active.toString()
        binding.note.text = area.note
        binding.type.text = area.type.toString()
        binding.title.text = area.name
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.admin_device_detail_info_options, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            R.id.admin_device_modify -> {
                val dialogLayout = AdminModifyDeviceDialogBinding.inflate(LayoutInflater.from(this))
                dialogLayout.owner.setText(device.owner.toString())
                dialogLayout.area.setText(device.area.toString())
                dialogLayout.active.setText(device.active.toString())
                dialogLayout.name.setText(device.name)
                dialogLayout.note.setText(device.note)

                AlertDialog.Builder(this)
                    .setTitle("修改信息")
                    .setView(dialogLayout.root)
                    .setPositiveButton("确认修改") { _, _ ->
                        val service = ServiceCreator.create(AdminService::class.java)
                        val data = AdminModifyDevice(
                            device.id,
                            dialogLayout.owner.text.toString().toLongOrNull(),
                            dialogLayout.area.text.toString().toLongOrNull(),
                            dialogLayout.name.text.toString(),
                            dialogLayout.active.text.toString().toBooleanStrict(),
                            dialogLayout.note.text.toString(),
                            dialogLayout.apSsid.text.toString(),
                            dialogLayout.apPassword.text.toString(),
                            dialogLayout.staSsid.text.toString(),
                            dialogLayout.staPassword.text.toString(),
                            dialogLayout.master.text.toString().toLongOrNull()
                        )
                        service.modifyDevice(LoginConfiguration.userInfo!!.token, data)
                            .enqueue(object :
                                Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        finish()
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .setNegativeButton("取消修改") { _, _ ->

                    }
                    .show()
            }
            R.id.admin_device_delete -> {
                ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@DeviceDetailInfoActivity)
                    .setContent("确定删除设备：${device.name}吗")
                    .setCancelText("取消删除")
                    .setEnsureText("确定删除")
                    .setEnsureListener {
                        val service = ServiceCreator.create(AdminService::class.java)
                        val rmList = ArrayList<Long>()
                        rmList.add(device.id)
                        val rmData = AdminDeleteDevice(rmList)
                        service.deleteDevice(LoginConfiguration.userInfo!!.token, rmData)
                            .enqueue(object :
                                Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        finish()
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .build()
                    .show()
            }
            R.id.admin_finger_enroll -> {
                val dialogLayout =
                    AdminEnrollFingerPrintDialogBinding.inflate(LayoutInflater.from(this))

                AlertDialog.Builder(this)
                    .setTitle("录入指纹")
                    .setView(dialogLayout.root)
                    .setPositiveButton("确定录入") { _, _ ->
                        val service = ServiceCreator.create(AdminService::class.java)
                        val data = EnrollFingerPrint(
                            dialogLayout.userId.text.toString().toLongOrNull(),
                            device.mac
                        )
                        service.enrollFingerPrint(LoginConfiguration.userInfo!!.token, data)
                            .enqueue(object :
                                Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        ResponseHandler.showMessage(it)
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .setNegativeButton("取消录入") { _, _ ->

                    }
                    .show()
            }
        }
        return true
    }
}