package org.brynhild.graduation.activity.admin

import android.app.AlertDialog
import android.os.Bundle
import android.os.Handler
import android.util.Log
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuItem
import androidx.recyclerview.widget.GridLayoutManager
import com.example.devicemanager.view.ConfirmPopupWindow
import com.google.gson.Gson
import org.brynhild.graduation.R
import org.brynhild.graduation.activity.common.BaseActivity
import org.brynhild.graduation.common.callback.RecyclerViewCallback
import org.brynhild.graduation.common.config.LoginConfiguration
import org.brynhild.graduation.common.transfer.Result
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.user.*
import org.brynhild.graduation.common.utils.JsonUtils
import org.brynhild.graduation.common.utils.MultipleSelectHandler
import org.brynhild.graduation.databinding.ActivityMajorDetailInfoBinding
import org.brynhild.graduation.databinding.AdminCreateClassDialogBinding
import org.brynhild.graduation.databinding.AdminModifyMajorDialogBinding
import org.brynhild.graduation.network.adapter.ClassEntityAdapter
import org.brynhild.graduation.network.entiity.ClassEntity
import org.brynhild.graduation.network.entiity.Major
import org.brynhild.graduation.service.http.AdminService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class MajorDetailInfoActivity : BaseActivity() {
    private lateinit var binding: ActivityMajorDetailInfoBinding
    private lateinit var major: Major
    private val refreshClassList = 0

    val handler: Handler = Handler {
        when (it.what) {
            refreshClassList -> {
                loadClassList()
            }
        }
        false
    }

    private fun loadClassList() {
        val service = ServiceCreator.create(AdminService::class.java)
        val data = QueryClassRequest(null, null, null, null)
        Log.d("Admin", "查询班级列表：${data}")
        service.queryClass(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "AddDeviceActivity",
                            "flag is ${result.flag},message is ${result.message},data is ${result.data}"
                        )
                        val classEntityArrayList = ArrayList<ClassEntity>()
                        val gson = Gson()
                        for (item in result.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val classEntity = JsonUtils.fromJson<ClassEntity>(json)
                            if (classEntity != null) {
                                classEntityArrayList.add(classEntity)
                            }
                        }
                        val adapter =
                            ClassEntityAdapter(this@MajorDetailInfoActivity, classEntityArrayList)

                        binding.recyclerView.adapter = MultipleSelectHandler.createHandler(
                            this@MajorDetailInfoActivity,
                            adapter,
                            object : RecyclerViewCallback {
                                override fun execute(selected: List<Int>) {
                                    val builder = StringBuilder()
                                    println(selected)
                                    var first = true

                                    val rmList = ArrayList<Long>()
                                    for (index in selected) {
                                        rmList.add(classEntityArrayList[index].id)
                                        if (!first) {
                                            builder.append("、")
                                        } else {
                                            first = false
                                        }
                                        builder.append(classEntityArrayList[index].name)
                                    }
                                    ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@MajorDetailInfoActivity)
                                        .setContent("以下班级将被删除：${builder}")
                                        .setCancelText("取消删除")
                                        .setEnsureText("确定删除")
                                        .setEnsureListener {

                                            val service =
                                                ServiceCreator.create(AdminService::class.java)
                                            val rmData = AdminDeleteClass(rmList)
                                            println("send data with:${Gson().toJson(rmData)}")
                                            service.deleteClass(
                                                LoginConfiguration.userInfo!!.token,
                                                rmData
                                            ).enqueue(object : Callback<Result> {
                                                override fun onResponse(
                                                    call: Call<Result>,
                                                    response: Response<Result>
                                                ) {
                                                    ResponseHandler.handle(response, {
                                                        handler.sendEmptyMessage(refreshClassList)
                                                    }, {
                                                        ResponseHandler.showMessage(it)
                                                    })
                                                }

                                                override fun onFailure(
                                                    call: Call<Result>,
                                                    t: Throwable
                                                ) {
                                                    t.printStackTrace()
                                                }
                                            })

                                        }
                                        .build()
                                        .show()
                                }
                            })
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityMajorDetailInfoBinding.inflate(LayoutInflater.from(this))
        setContentView(binding.root)
        setSupportActionBar(binding.toolBar)
        val layoutManager = GridLayoutManager(this, 1)
        binding.recyclerView.layoutManager = layoutManager

        val gson = Gson()
        major = gson.fromJson(intent.getStringExtra("major"), Major::class.java)

        setTableData(major)

        bindOnClick()

        loadClassList()
    }

    private fun bindOnClick() {
        binding.backButton.setOnClickListener {
            finish()
        }
    }

    private fun setTableData(major: Major) {
        binding.code.text = major.code
        binding.name.text = major.name
        binding.title.text = major.name
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.admin_major_detail_info_options, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            R.id.admin_major_modify -> {
                val dialogLayout = AdminModifyMajorDialogBinding.inflate(LayoutInflater.from(this))
                dialogLayout.code.setText(major.code)
                dialogLayout.name.setText(major.name)

                AlertDialog.Builder(this)
                    .setTitle("修改信息")
                    .setView(dialogLayout.root)
                    .setPositiveButton("确认修改") { _, _ ->
                        val service = ServiceCreator.create(AdminService::class.java)
                        val data = AdminModifyMajor(
                            major.id,
                            dialogLayout.name.text.toString(),
                            dialogLayout.code.text.toString(),
                        )
                        service.modifyMajor(LoginConfiguration.userInfo!!.token, data)
                            .enqueue(object :
                                Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        finish()
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .setNegativeButton("取消修改") { _, _ ->

                    }
                    .show()
            }
            R.id.admin_major_delete -> {
                ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@MajorDetailInfoActivity)
                    .setContent("确定删除专业：${major.name}吗")
                    .setCancelText("取消删除")
                    .setEnsureText("确定删除")
                    .setEnsureListener {
                        val service = ServiceCreator.create(AdminService::class.java)
                        val rmList = ArrayList<Long>()
                        rmList.add(major.id)
                        val rmData = AdminDeleteMajor(rmList)
                        service.deleteMajor(LoginConfiguration.userInfo!!.token, rmData)
                            .enqueue(object :
                                Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        finish()
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .build()
                    .show()


            }
            R.id.admin_class_add -> {
                val dialogLayout = AdminCreateClassDialogBinding.inflate(LayoutInflater.from(this))

                AlertDialog.Builder(this)
                    .setTitle("添加班级")
                    .setView(dialogLayout.root)
                    .setPositiveButton("确认添加") { _, _ ->
                        val service = ServiceCreator.create(AdminService::class.java)
                        val data = AdminAddClass(
                            dialogLayout.no.text.toString(),
                            dialogLayout.name.text.toString(),
                            major.id,
                            major.academyId
                        )
                        service.addClass(LoginConfiguration.userInfo!!.token, data)
                            .enqueue(object : Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        handler.sendEmptyMessage(refreshClassList)
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .setNegativeButton("取消添加") { _, _ ->

                    }
                    .show()
            }
        }
        return true
    }
}