package org.brynhild.graduation.activity.admin

import android.app.Activity
import android.app.AlertDialog
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.os.Message
import android.view.LayoutInflater
import android.view.Menu
import android.view.MenuItem
import android.view.View
import android.widget.Toast
import androidx.recyclerview.widget.GridLayoutManager
import com.bumptech.glide.Glide
import com.example.devicemanager.view.ConfirmPopupWindow
import com.google.gson.Gson
import org.brynhild.graduation.R
import org.brynhild.graduation.activity.common.BaseActivity
import org.brynhild.graduation.activity.common.StatisticActivity
import org.brynhild.graduation.common.config.LoginConfiguration
import org.brynhild.graduation.common.config.MyApplication
import org.brynhild.graduation.common.constant.AccountConstant
import org.brynhild.graduation.common.constant.BroadcastConstant
import org.brynhild.graduation.common.constant.StatisticConstant
import org.brynhild.graduation.common.constant.UserStateConstant
import org.brynhild.graduation.common.transfer.Result
import org.brynhild.graduation.common.transfer.dto.FileInfo
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.device.QuerySignInRecordRequest
import org.brynhild.graduation.common.transfer.vo.user.AdminDeleteUser
import org.brynhild.graduation.common.transfer.vo.user.AdminModifyUser
import org.brynhild.graduation.common.utils.FileUploadUtil
import org.brynhild.graduation.common.utils.JsonUtils
import org.brynhild.graduation.common.utils.Uri2PathUtil
import org.brynhild.graduation.databinding.ActivityUserDetailInfoBinding
import org.brynhild.graduation.databinding.AdminModifyUserDialogBinding
import org.brynhild.graduation.network.adapter.SignInRecordAdapter
import org.brynhild.graduation.network.entiity.SignInRecord
import org.brynhild.graduation.network.entiity.User
import org.brynhild.graduation.service.http.AdminService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import java.io.File
import java.io.IOException

class UserDetailInfoActivity : BaseActivity() {
    private lateinit var binding: ActivityUserDetailInfoBinding
    private lateinit var user: User
    private val updateAccountAvatar = 2
    private val loadSignInRecord = 1

    val handler: Handler = Handler { it ->
        when (it.what) {
            updateAccountAvatar -> {
                val path = it.obj as String
                Glide.with(this).load(path).into(binding.iconImage)
                val service = ServiceCreator.create(AdminService::class.java)
                val data = AdminModifyUser(
                    user.id,
                    null,
                    path,
                    null,
                    null,
                    null,
                    null,
                    null,
                    null,
                    null,
                    null
                )
                service.modifyUser(LoginConfiguration.userInfo!!.token, data)
                    .enqueue(object : Callback<Result> {
                        override fun onResponse(call: Call<Result>, response: Response<Result>) {
                            ResponseHandler.handle(response, { ret ->
                                ResponseHandler.showMessage(ret)
                            }, { ret ->
                                ResponseHandler.showMessage(ret)
                            })
                        }

                        override fun onFailure(call: Call<Result>, t: Throwable) {
                            Toast.makeText(MyApplication.context, "请求失败", Toast.LENGTH_SHORT).show()
                        }
                    })
            }
            loadSignInRecord -> {
                val service = ServiceCreator.create(AdminService::class.java)
                val data = QuerySignInRecordRequest(user.id, null, null, null, null, null)
                service.querySignInRecord(LoginConfiguration.userInfo!!.token, data)
                    .enqueue(object : Callback<Result> {
                        override fun onResponse(call: Call<Result>, response: Response<Result>) {
                            ResponseHandler.handle(response, { result ->
                                val recordArrayList = ArrayList<SignInRecord>()
                                val gson = Gson()
                                for (item in result.data as ArrayList<*>) {
                                    val json = gson.toJson(item)
                                    println(json)
                                    val record = JsonUtils.fromJson<SignInRecord>(json)
                                    if (record != null) {
                                        recordArrayList.add(record)
                                    }
                                }
                                val adapter = SignInRecordAdapter(
                                    this@UserDetailInfoActivity,
                                    recordArrayList
                                )
                                binding.recordView.adapter = adapter
                            }, {
                                ResponseHandler.showMessage(it)
                            })
                        }

                        override fun onFailure(call: Call<Result>, t: Throwable) {
                            t.printStackTrace()
                        }
                    })
            }
        }
        false
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityUserDetailInfoBinding.inflate(LayoutInflater.from(this))
        setContentView(binding.root)
        setSupportActionBar(binding.toolBar)

        val gson = Gson()
        user = gson.fromJson(intent.getStringExtra("user"), User::class.java)

        setTableData(user)

        bindOnClick()

        binding.recordView.layoutManager = GridLayoutManager(this, 1)

        handler.sendEmptyMessage(loadSignInRecord)

    }

    private fun setTableData(user: User) {
        binding.title.text = user.name

        binding.username.text = user.name
        binding.sex.text = user.sex
        binding.email.text = user.email
        binding.tel.text = user.tel
        binding.no.text = user.no
        user.type?.let {
            binding.type.text = when (it) {
                AccountConstant.USER -> {
                    "学生"
                }
                AccountConstant.TEACHER -> {
                    "教师"
                }
                AccountConstant.ADMIN -> {
                    "管理员"
                }
                else -> {
                    "错误的类型"
                }
            }
        }
        binding.state.text = when (user.state) {
            UserStateConstant.NOT_ACTIVE -> {
                "未激活"
            }
            UserStateConstant.READY -> {
                "正常"
            }
            UserStateConstant.FORBIDDEN -> {
                "已禁用"
            }
            else -> {
                "错误的状态"
            }
        }
        binding.lastLogin.text = user.lastLogin

        Glide.with(this).load(user.avatar ?: R.drawable.nav_user).into(binding.iconImage)
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (requestCode) {
            updateAccountAvatar -> {
                if (resultCode == Activity.RESULT_OK && data != null) {
                    val uri = data.data
                    if (Uri2PathUtil.getRealPathFromUri(this, uri) != null) {
                        //从uri得到绝对路径，并获取到file文件
                        val file = File(Uri2PathUtil.getRealPathFromUri(this, uri))
                        FileUploadUtil.uploadFile(file, object : okhttp3.Callback {
                            override fun onFailure(call: okhttp3.Call, e: IOException) {
                                println("failed")
                            }

                            override fun onResponse(
                                call: okhttp3.Call,
                                response: okhttp3.Response
                            ) {
                                if (response.isSuccessful) {
                                    val body = response.body()?.string()
                                    val gson = Gson()
                                    val result = gson.fromJson(body, Result::class.java)
                                    println(result)
                                    var json = gson.toJson(result.data)
                                    val fileInfoList = JsonUtils.fromJson2List<FileInfo>(json)
                                    if (fileInfoList?.size == 1) {
                                        val message = Message()
                                        message.what = updateAccountAvatar
                                        json = gson.toJson(fileInfoList[0])
                                        val info = JsonUtils.fromJson<FileInfo>(json)
                                        println(info)
                                        if (info != null) {
                                            message.obj = info.path
                                            handler.sendMessage(message)
                                        }
                                    }
                                }
                            }
                        })
                    } else {
                        Toast.makeText(this, "获取文件失败", Toast.LENGTH_SHORT).show()
                    }
                }
            }
        }
    }

    private fun bindOnClick() {
        binding.backButton.setOnClickListener {
            finish()
        }

        binding.infoBtn.setOnClickListener {
            binding.infoLayout.visibility = View.VISIBLE
            binding.recordLayout.visibility = View.GONE
        }

        binding.recordBtn.setOnClickListener {
            binding.infoLayout.visibility = View.GONE
            binding.recordLayout.visibility = View.VISIBLE
        }

        binding.iconImage.setOnClickListener {
            val intent = Intent(Intent.ACTION_OPEN_DOCUMENT)
            intent.addCategory(Intent.CATEGORY_OPENABLE)
            intent.type = "image/*"
            startActivityForResult(intent, updateAccountAvatar)
        }

        binding.statisticBtn.setOnClickListener {
            val intent = Intent(this, StatisticActivity::class.java)
            intent.putExtra(StatisticConstant.ACTION, StatisticConstant.STUDENT_ACTION)
            intent.putExtra(StatisticConstant.ID_ENUM, user.id)
            startActivity(intent)
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.admin_user_detail_info_options, menu)
        return true
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        when (item.itemId) {
            R.id.admin_user_modify -> {
                val dialogLayout = AdminModifyUserDialogBinding.inflate(LayoutInflater.from(this))
                dialogLayout.email.setText(user.email)
                dialogLayout.name.setText(user.name)
                dialogLayout.sex.setText(user.sex)
                dialogLayout.tel.setText(user.tel)
                dialogLayout.departmentNo.setText("${user.departmentId}")


                dialogLayout.type.setText( when (user.type) {
                    AccountConstant.USER -> {
                        "学生"
                    }
                    AccountConstant.TEACHER -> {
                        "教师"
                    }
                    AccountConstant.ADMIN -> {
                        "管理员"
                    }
                    else -> {
                        "错误的类型"
                    }
                })

                dialogLayout.state.setText(when (user.state) {
                    UserStateConstant.NOT_ACTIVE -> {
                        "未激活"
                    }
                    UserStateConstant.READY -> {
                        "正常"
                    }
                    UserStateConstant.FORBIDDEN -> {
                        "已禁用"
                    }
                    else -> {
                        "错误的状态"
                    }
                })

                AlertDialog.Builder(this)
                    .setTitle("修改信息")
                    .setView(dialogLayout.root)
                    .setPositiveButton("确认修改") { _, _ ->
                        val service = ServiceCreator.create(AdminService::class.java)
                        val data = AdminModifyUser(
                            user.id,
                            dialogLayout.password.text.toString(),
                            null,
                            dialogLayout.email.text.toString(),
                            null,
                            null,
                            dialogLayout.name.text.toString(),
                            dialogLayout.sex.text.toString(),
                            null,
                            dialogLayout.tel.text.toString(),
                            dialogLayout.departmentNo.text.toString()
                        )
                        service.modifyUser(LoginConfiguration.userInfo!!.token, data)
                            .enqueue(object : Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        finish()
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .setNegativeButton("取消修改") { _, _ ->

                    }
                    .show()
            }
            R.id.admin_user_delete -> {
                ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@UserDetailInfoActivity)
                    .setContent("确定删除用户：${user.name}吗")
                    .setCancelText("取消删除")
                    .setEnsureText("确定删除")
                    .setEnsureListener {
                        val service = ServiceCreator.create(AdminService::class.java)
                        val rmList = ArrayList<Long>()
                        rmList.add(user.id)
                        val rmData = AdminDeleteUser(rmList)
                        service.deleteUser(LoginConfiguration.userInfo!!.token, rmData)
                            .enqueue(object : Callback<Result> {
                                override fun onResponse(
                                    call: Call<Result>,
                                    response: Response<Result>
                                ) {
                                    ResponseHandler.handle(response, {
                                        finish()
                                        if (LoginConfiguration.userInfo!!.user.id == user.id) {
                                            val intent = Intent(BroadcastConstant.OFFLINE)
                                            MyApplication.context.sendBroadcast(intent)
                                        }
                                    }, {
                                        ResponseHandler.showMessage(it)
                                    })
                                }

                                override fun onFailure(call: Call<Result>, t: Throwable) {
                                    t.printStackTrace()
                                }
                            })
                    }
                    .build()
                    .show()
            }
        }
        return true
    }
}