package org.brynhild.graduation.activity.common

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.os.Bundle
import android.os.PersistableBundle
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.appcompat.app.AppCompatActivity
import org.brynhild.graduation.activity.ActivityCollector
import org.brynhild.graduation.activity.admin.AdminMainPageActivity
import org.brynhild.graduation.activity.teacher.TeacherMainPageActivity
import org.brynhild.graduation.activity.user.UserActivity
import org.brynhild.graduation.common.config.LoginConfiguration
import org.brynhild.graduation.common.config.MyApplication
import org.brynhild.graduation.common.constant.AccountConstant
import org.brynhild.graduation.common.constant.BroadcastConstant
import org.brynhild.graduation.service.mqtt.MqttListenService


open class BaseActivity : AppCompatActivity() {

    lateinit var receiver: ForceOfflineReceiver
    lateinit var loginSuccessReceiver: LoginSuccessReceiver
    lateinit var loginFailedReceiver: LoginFailedReceiver

    override fun onCreate(savedInstanceState: Bundle?, persistentState: PersistableBundle?) {
        super.onCreate(savedInstanceState, persistentState)
        ActivityCollector.addActivity(this)
    }

    override fun onResume() {
        super.onResume()
        val intentFilter = IntentFilter()
        intentFilter.addAction(BroadcastConstant.OFFLINE)
        receiver = ForceOfflineReceiver()
        registerReceiver(receiver, intentFilter)

        val loginSuccessFilter = IntentFilter()
        loginSuccessFilter.addAction(BroadcastConstant.LOGIN_SUCCESS)
        loginSuccessReceiver = LoginSuccessReceiver()
        registerReceiver(loginSuccessReceiver, loginSuccessFilter)

        val loginFailedFilter = IntentFilter()
        loginFailedFilter.addAction(BroadcastConstant.LOGIN_FAILED)
        loginFailedReceiver = LoginFailedReceiver()
        registerReceiver(loginFailedReceiver, loginFailedFilter)

    }

    override fun onPause() {
        super.onPause()
        unregisterReceiver(receiver)
        unregisterReceiver(loginSuccessReceiver)
        unregisterReceiver(loginFailedReceiver)
    }

    override fun onDestroy() {
        super.onDestroy()
        ActivityCollector.removeActivity(this)
    }

    inner class ForceOfflineReceiver : BroadcastReceiver() {
        override fun onReceive(context: Context, intent: Intent) {
            AlertDialog.Builder(context).apply {
                setTitle("Warning")
                setMessage("You are forced to be offline.Please try to login again.")
                setCancelable(false)
                setPositiveButton("OK") { _, _ ->
                    finishAffinity()
                    val serviceIntent = Intent(MyApplication.context, MqttListenService::class.java)
                    stopService(serviceIntent)
                    val i = Intent(context, LoginActivity::class.java)
                    context.startActivity(i)
                }
                show()
            }
        }
    }

    inner class LoginSuccessReceiver : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent) {
            val serviceIntent = Intent(MyApplication.context, MqttListenService::class.java)
            startService(serviceIntent)

            var activityIntent: Intent? = null
            when (LoginConfiguration.userInfo!!.user.type) {
                AccountConstant.USER -> {
                    activityIntent = Intent(MyApplication.context, UserActivity::class.java)
                }
                AccountConstant.ADMIN -> {
                    activityIntent =
                        Intent(MyApplication.context, AdminMainPageActivity::class.java)
                }
                AccountConstant.TEACHER -> {
                    activityIntent =
                        Intent(MyApplication.context, TeacherMainPageActivity::class.java)
                }
                else -> {
                    Toast.makeText(MyApplication.context, "用户类型异常，请联系管理员", Toast.LENGTH_SHORT)
                        .show()
                }
            }
            if (activityIntent != null) {
                startActivity(activityIntent)
            }
        }
    }

    inner class LoginFailedReceiver : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent) {
            val intent = Intent(MyApplication.context, LoginActivity::class.java)
            startActivity(intent)
        }
    }
}