package org.brynhild.graduation.activity.common

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.widget.Toast
import androidx.lifecycle.ViewModelProvider
import org.brynhild.graduation.activity.common.view.model.ForgetPasswordViewModel
import org.brynhild.graduation.common.config.MyApplication
import org.brynhild.graduation.common.transfer.Result
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.user.ResendEmail
import org.brynhild.graduation.databinding.ActivityForgetPasswordBinding
import org.brynhild.graduation.service.http.AccountService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class ForgetPasswordActivity : BaseActivity() {

    lateinit var viewModel: ForgetPasswordViewModel

    private lateinit var activityForgetPasswordBinding: ActivityForgetPasswordBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        supportActionBar?.hide()
        activityForgetPasswordBinding =
            ActivityForgetPasswordBinding.inflate(LayoutInflater.from(this))
        setContentView(activityForgetPasswordBinding.root)
        bindObserver()
        bindOnClick()
    }

    private fun bindObserver() {
        viewModel = ViewModelProvider(this)[ForgetPasswordViewModel::class.java]

        activityForgetPasswordBinding.username.setOnFocusChangeListener { _, hasFocus ->
            if (!hasFocus) {
                viewModel.setUsername(activityForgetPasswordBinding.username.text.toString())
            }
        }
        activityForgetPasswordBinding.email.setOnFocusChangeListener { _, hasFocus ->
            if (!hasFocus) {
                viewModel.setEmail(activityForgetPasswordBinding.email.text.toString())
            }
        }


        viewModel.username.observe(this) { accountName ->
            activityForgetPasswordBinding.username.setText(accountName)
        }
        viewModel.email.observe(this) { code ->
            activityForgetPasswordBinding.email.setText(code)
        }
    }

    private fun bindOnClick() {
        activityForgetPasswordBinding.resetButton.setOnClickListener {
            val service = ServiceCreator.create(AccountService::class.java)
            val data = ResendEmail(
                activityForgetPasswordBinding.username.text.toString(),
                activityForgetPasswordBinding.email.text.toString(), 2
            )
            service.resendEmail(data).enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    Log.d("Forget", response.raw().toString())
                    ResponseHandler.handle(response, {
                        Toast.makeText(
                            this@ForgetPasswordActivity,
                            "${it.message}",
                            Toast.LENGTH_SHORT
                        ).show()
                    }, {
                        Toast.makeText(
                            this@ForgetPasswordActivity,
                            "${it.message}",
                            Toast.LENGTH_SHORT
                        ).show()
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    Toast.makeText(MyApplication.context, "网络错误", Toast.LENGTH_SHORT).show()
                }
            })
        }
    }
}