package org.brynhild.graduation.activity.common

import android.content.Context
import android.content.Intent
import android.content.SharedPreferences
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.widget.Toast
import androidx.lifecycle.ViewModelProvider
import com.google.gson.Gson
import org.brynhild.graduation.activity.common.view.factory.LoginViewModelFactory
import org.brynhild.graduation.activity.common.view.model.LoginViewModel
import org.brynhild.graduation.common.config.LoginConfiguration
import org.brynhild.graduation.common.config.MyApplication
import org.brynhild.graduation.common.constant.BroadcastConstant
import org.brynhild.graduation.common.transfer.Result
import org.brynhild.graduation.common.transfer.dto.UserInfo
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.user.AccountLoginInfo
import org.brynhild.graduation.common.utils.JsonUtils
import org.brynhild.graduation.databinding.ActivityLoginBinding
import org.brynhild.graduation.service.http.AccountService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class LoginActivity : BaseActivity() {

    lateinit var viewModel: LoginViewModel
    lateinit var sp: SharedPreferences
    private var lastBack = 0L
    private lateinit var activityLoginBinding: ActivityLoginBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        supportActionBar?.hide()
        activityLoginBinding = ActivityLoginBinding.inflate(LayoutInflater.from(this))
        setContentView(activityLoginBinding.root)

        sp = getSharedPreferences("account", Context.MODE_PRIVATE)
        var accountName = sp.getString("accountName", "")
        if (accountName == null) {
            accountName = ""
        }
        viewModel = ViewModelProvider(
            this,
            LoginViewModelFactory(accountName, "")
        )[LoginViewModel::class.java]

        activityLoginBinding.loginButton.setOnClickListener {
            val accountService = ServiceCreator.create(AccountService::class.java)
            val loginInfo =
                AccountLoginInfo(
                    activityLoginBinding.accountName.text.toString(),
                    activityLoginBinding.accountPassword.text.toString()
                )
            accountService.login(loginInfo).enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "LoginActivity",
                            "flag is ${result.flag},message is ${result.message},token is ${result.data}"
                        )
                        val json = Gson().toJson(result.data)

                        LoginConfiguration.userInfo = JsonUtils.fromJson<UserInfo>(json)
                        LoginConfiguration.login_username = loginInfo.username
                        LoginConfiguration.login_password = loginInfo.password
                        saveLoginInfo()
                        val b = Intent(BroadcastConstant.LOGIN_SUCCESS)
                        MyApplication.context.sendBroadcast(b)
                        finish()
                    }, { result ->
                        Log.d(
                            "LoginActivity",
                            "flag is ${result.flag},message is ${result.message}"
                        )
                        Toast.makeText(this@LoginActivity, result.message, Toast.LENGTH_SHORT)
                            .show()
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
        }

        activityLoginBinding.registerButton.setOnClickListener {
            val intent = Intent(this, RegisterActivity::class.java)
            startActivity(intent)
        }

        activityLoginBinding.forgetPasswordButton.setOnClickListener {
            val intent = Intent(this, ForgetPasswordActivity::class.java)
            startActivity(intent)
        }

        activityLoginBinding.accountName.setOnFocusChangeListener { _, hasFocus ->
            if (!hasFocus) {
                viewModel.setAccountName(activityLoginBinding.accountName.text.toString())
            }
        }

        viewModel.accountName.observe(this) { accountName ->
            activityLoginBinding.accountName.setText(accountName)
        }

        activityLoginBinding.accountPassword.setOnFocusChangeListener { _, hasFocus ->
            if (!hasFocus) {
                viewModel.setAccountPassword(activityLoginBinding.accountPassword.text.toString())
            }
        }

        viewModel.accountPassword.observe(this) { accountPassword ->
            activityLoginBinding.accountPassword.setText(accountPassword)
        }

        activityLoginBinding.readUserPolicy.setOnClickListener {
            val intent = Intent(this, PolicyActivity::class.java).apply {
                putExtra("type", PolicyActivity.USER_POLICY)
            }
            startActivity(intent)
        }

        activityLoginBinding.readPrivacyPolicy.setOnClickListener {
            val intent = Intent(this, PolicyActivity::class.java).apply {
                putExtra("type", PolicyActivity.PRIVACY_POLICY)
            }
            startActivity(intent)
        }
    }

    private fun saveLoginInfo() {
        val accountName = LoginConfiguration.login_username
        val accountPassword = LoginConfiguration.login_password
        with(sp.edit()) {
            putString("accountName", accountName)
            putString("accountPassword", accountPassword)
            apply()
        }
        Log.d("LoginActivity", "save info:name:$accountName,password:$accountPassword")
    }


    override fun onBackPressed() {
        if (lastBack == 0L || System.currentTimeMillis() - lastBack > 2000) {
            Toast.makeText(this, "再按一次退出程序", Toast.LENGTH_SHORT).show()
            lastBack = System.currentTimeMillis()
        } else {
            finish()
        }
    }
}