package org.brynhild.graduation.activity.common

import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import androidx.lifecycle.ViewModelProvider
import com.example.devicemanager.view.ConfirmPopupWindow
import org.brynhild.graduation.activity.common.view.model.RegisterViewModel
import org.brynhild.graduation.common.transfer.Result
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.user.AccountRegisterInfo
import org.brynhild.graduation.databinding.ActivityRegisterBinding
import org.brynhild.graduation.service.http.AccountService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class RegisterActivity : BaseActivity() {

    lateinit var viewModel: RegisterViewModel
    private lateinit var activityRegisterBinding: ActivityRegisterBinding

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
//        setContentView(R.layout.activity_register)
        supportActionBar?.hide()
        activityRegisterBinding = ActivityRegisterBinding.inflate(LayoutInflater.from(this))
        setContentView(activityRegisterBinding.root)
        viewModel = ViewModelProvider(this)[RegisterViewModel::class.java]

        activityRegisterBinding.username.setOnFocusChangeListener { _, hasFocus ->
            if (!hasFocus) {
                viewModel.setAccountName(activityRegisterBinding.username.text.toString())
            }
        }
        viewModel.accountName.observe(this) { accountName ->
            activityRegisterBinding.username.setText(accountName)
        }

        activityRegisterBinding.password.setOnFocusChangeListener { _, hasFocus ->
            if (!hasFocus) {
                viewModel.setAccountPassword(activityRegisterBinding.password.text.toString())
            }
        }
        viewModel.accountPassword.observe(this) { accountPassword ->
            activityRegisterBinding.password.setText(accountPassword)
        }

        activityRegisterBinding.agreePolicy.setOnFocusChangeListener { _, hasFocus ->
            if (!hasFocus) {
                viewModel.setAgree(activityRegisterBinding.agreePolicy.isChecked)
            }
        }
        viewModel.agree.observe(this) { agree ->
            activityRegisterBinding.agreePolicy.isChecked = agree
        }

        activityRegisterBinding.email.setOnFocusChangeListener { _, hasFocus ->
            if (!hasFocus) {
                viewModel.setEmail(activityRegisterBinding.email.text.toString())
            }
        }
        viewModel.email.observe(this) { email ->
            activityRegisterBinding.email.setText(email)
        }

        activityRegisterBinding.name.setOnFocusChangeListener { _, hasFocus ->
            if (!hasFocus) {
                viewModel.setName(activityRegisterBinding.name.text.toString())
            }
        }
        viewModel.name.observe(this) { name ->
            activityRegisterBinding.name.setText(name)
        }

        activityRegisterBinding.tel.setOnFocusChangeListener { _, hasFocus ->
            if (!hasFocus) {
                viewModel.setTel(activityRegisterBinding.tel.text.toString())
            }
        }
        viewModel.tel.observe(this) { tel ->
            activityRegisterBinding.tel.setText(tel)
        }

        activityRegisterBinding.no.setOnFocusChangeListener { _, hasFocus ->
            if (!hasFocus) {
                viewModel.setNo(activityRegisterBinding.no.text.toString())
            }
        }
        viewModel.no.observe(this) { no ->
            activityRegisterBinding.no.setText(no)
        }

        activityRegisterBinding.readUserPolicy.setOnClickListener {
            val intent = Intent(this, PolicyActivity::class.java).apply {
                putExtra("type", PolicyActivity.USER_POLICY)
            }
            startActivity(intent)
        }

        activityRegisterBinding.readPrivacyPolicy.setOnClickListener {
            val intent = Intent(this, PolicyActivity::class.java).apply {
                putExtra("type", PolicyActivity.PRIVACY_POLICY)
            }
            startActivity(intent)
        }

        activityRegisterBinding.registerButton.setOnClickListener {
            if (!activityRegisterBinding.agreePolicy.isChecked) {
                ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@RegisterActivity)
                    .setContent("请阅读并同意隐私政策与用户协议")
                    .build().show()
            }

            val accountService = ServiceCreator.create(AccountService::class.java)
            val registerInfo = AccountRegisterInfo(
                activityRegisterBinding.username.text.toString(),
                activityRegisterBinding.password.text.toString(),
                activityRegisterBinding.email.text.toString(),
                activityRegisterBinding.name.text.toString(),
                activityRegisterBinding.tel.text.toString(),
                activityRegisterBinding.no.text.toString(),
            )
            accountService.accountRegister(registerInfo).enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "RegisterActivity",
                            "flag is ${result.flag},message is ${result.message},data is ${result.data}"
                        )
                        ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@RegisterActivity)
                            .setContent("注册成功，请前往邮箱，并点击邮箱中的激活链接完成激活")
                            .setCancelText("取消")
                            .setEnsureText("返回登录页")
                            .setEnsureListener {
                                val intent =
                                    Intent(this@RegisterActivity, LoginActivity::class.java)
                                startActivity(intent)
                            }
                            .build().show()
                    }, { result ->
                        Log.d(
                            "RegisterActivity",
                            "flag is ${result.flag},message is ${result.message}"
                        )
                        ConfirmPopupWindow.ConfirmPopupWindowBuilder.init(this@RegisterActivity)
                            .setContent("注册失败：${result.message}")
                            .setCancelText("取消")
                            .setEnsureText("确定")
                            .build().show()
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
        }
    }
}