package org.brynhild.graduation.activity.teacher

import android.os.Bundle
import android.os.Handler
import android.util.Log
import android.view.LayoutInflater
import androidx.recyclerview.widget.GridLayoutManager
import com.google.gson.Gson
import org.brynhild.graduation.activity.common.BaseActivity
import org.brynhild.graduation.common.config.LoginConfiguration
import org.brynhild.graduation.common.transfer.Result
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.user.QueryUserRequest
import org.brynhild.graduation.common.utils.JsonUtils
import org.brynhild.graduation.databinding.ActivityClassDetailInfoUsingByTeacherBinding
import org.brynhild.graduation.network.adapter.UserAdapterUsingByTeacher
import org.brynhild.graduation.network.entiity.ClassEntity
import org.brynhild.graduation.network.entiity.User
import org.brynhild.graduation.service.http.TeacherService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response


class ClassDetailInfoActivityUsingByTeacher : BaseActivity() {
    private lateinit var binding: ActivityClassDetailInfoUsingByTeacherBinding
    private lateinit var classEntity: ClassEntity
    private val refreshStudentList = 0
    private var continueEnroll = true
    private var enrollDeviceMac = ""
    private lateinit var userList: ArrayList<User>

    val handler: Handler = Handler {
        when (it.what) {
            refreshStudentList -> {
                loadStudentList()
            }
        }
        false
    }

    private fun loadStudentList() {
        val service = ServiceCreator.create(TeacherService::class.java)
        val data = QueryUserRequest(null, null, null, null, null, null, null, classEntity.id)
        Log.d("Admin", "查询班级列表：${data}")
        service.queryUser(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "AddDeviceActivity",
                            "flag is ${result.flag},message is ${result.message},data is ${result.data}"
                        )
                        val userArrayList = ArrayList<User>()
                        val gson = Gson()
                        for (item in result.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val user = JsonUtils.fromJson<User>(json)
                            if (user != null) {
                                userArrayList.add(user)
                            }
                        }
                        userList = userArrayList
                        val adapter = UserAdapterUsingByTeacher(
                            this@ClassDetailInfoActivityUsingByTeacher,
                            userArrayList
                        )

                        binding.recyclerView.adapter = adapter
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        binding = ActivityClassDetailInfoUsingByTeacherBinding.inflate(LayoutInflater.from(this))
        setContentView(binding.root)
        setSupportActionBar(binding.toolBar)
        val layoutManager = GridLayoutManager(this, 2)
        binding.recyclerView.layoutManager = layoutManager

        val gson = Gson()
        classEntity = gson.fromJson(intent.getStringExtra("class"), ClassEntity::class.java)

        setTableData(classEntity)

        bindClick()

        loadStudentList()
    }

    private fun bindClick() {
        binding.backButton.setOnClickListener {
            finish()
        }
    }

    private fun setTableData(entity: ClassEntity) {
        binding.no.text = entity.no
        binding.name.text = entity.name
        binding.title.text = entity.name
    }
}