package org.brynhild.graduation.activity.teacher.ui.clazz

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.fragment.app.Fragment
import androidx.recyclerview.widget.GridLayoutManager
import com.google.gson.Gson
import org.brynhild.graduation.common.callback.DefaultCallback
import org.brynhild.graduation.common.config.LoginConfiguration
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.timetable.QueryTeachingClassRequest
import org.brynhild.graduation.common.utils.JsonUtils
import org.brynhild.graduation.databinding.FragmentClazzBinding
import org.brynhild.graduation.network.adapter.ClassEntityAdapterUsingByTeacher
import org.brynhild.graduation.network.entiity.ClassEntity
import org.brynhild.graduation.service.http.TeacherService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class ClazzFragment : Fragment() {

    private var _binding: FragmentClazzBinding? = null
    private val singleColumnLayout = GridLayoutManager(this@ClazzFragment.context, 1)

    // This property is only valid between onCreateView and
    // onDestroyView.
    private val binding get() = _binding!!

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {

        _binding = FragmentClazzBinding.inflate(inflater, container, false)
        val root: View = binding.root

        binding.recyclerView.layoutManager = singleColumnLayout

        binding.swipeRefresh.setOnRefreshListener {
            loadTeachingClassList {
                Toast.makeText(this@ClazzFragment.context, "数据更新完成", Toast.LENGTH_SHORT).show()
            }
            binding.swipeRefresh.isRefreshing = false
        }

        loadTeachingClassList {

        }

        return root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    private fun loadTeachingClassList(callback: DefaultCallback) {
        val service = ServiceCreator.create(TeacherService::class.java)
        val data = QueryTeachingClassRequest(LoginConfiguration.userInfo!!.user.id)
        service.queryTeachingClass(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<org.brynhild.graduation.common.transfer.Result> {
                override fun onResponse(
                    call: Call<org.brynhild.graduation.common.transfer.Result>,
                    response: Response<org.brynhild.graduation.common.transfer.Result>
                ) {
                    ResponseHandler.handle(response, {
                        val gson = Gson()
                        val clientArrayList = ArrayList<ClassEntity>()
                        for (item in it.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val user = JsonUtils.fromJson<ClassEntity>(json)
                            if (user != null) {
                                clientArrayList.add(user)
                            }
                        }
                        this@ClazzFragment.context?.let {
                            val adapter = ClassEntityAdapterUsingByTeacher(it, clientArrayList)
                            binding.recyclerView.adapter = adapter
                        }
                        callback.execute()
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(
                    call: Call<org.brynhild.graduation.common.transfer.Result>,
                    t: Throwable
                ) {
                    t.printStackTrace()
                }
            })
    }
}