package org.brynhild.graduation.activity.user.ui.sign

import android.annotation.SuppressLint
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.webkit.WebSettings
import android.webkit.WebView
import android.webkit.WebViewClient
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.google.gson.Gson
import org.brynhild.graduation.common.config.LoginConfiguration
import org.brynhild.graduation.common.config.MyApplication
import org.brynhild.graduation.common.transfer.Result
import org.brynhild.graduation.common.transfer.bo.timetable.StatisticInfo
import org.brynhild.graduation.common.transfer.dto.FileInfo
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.timetable.AdminQueryStudentStatistic
import org.brynhild.graduation.common.utils.FileDownloadUtil
import org.brynhild.graduation.common.utils.JsonUtils
import org.brynhild.graduation.databinding.FragmentSignInBinding
import org.brynhild.graduation.service.http.CommonService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class SignInFragment : Fragment() {

    private var _binding: FragmentSignInBinding? = null

    // This property is only valid between onCreateView and
    // onDestroyView.
    private val binding get() = _binding!!

    @SuppressLint("SetJavaScriptEnabled")
    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {

        _binding = FragmentSignInBinding.inflate(inflater, container, false)
        val root: View = binding.root

        val webSettings: WebSettings = binding.backupChart.settings
        webSettings.domStorageEnabled = true
        webSettings.javaScriptEnabled = true
        webSettings.javaScriptCanOpenWindowsAutomatically = true
        webSettings.setSupportZoom(false)
        webSettings.displayZoomControls = false
        binding.backupChart.setBackgroundColor(
            ContextCompat.getColor(
                MyApplication.context,
                android.R.color.transparent
            )
        )
        binding.backupChart.loadUrl("file:///android_asset/echarts.html")
        binding.backupChart.webViewClient = object : WebViewClient() {
            override fun onPageFinished(view: WebView?, url: String?) {
                super.onPageFinished(view, url)
                loadStatisticInfo()
            }
        }

        bindOnClick()

        return root
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    private fun bindOnClick() {
        binding.downloadStatistic.setOnClickListener {
            val service = ServiceCreator.create(CommonService::class.java)
            val handler = object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, {
                        val gson = Gson()
                        val list = ArrayList<FileInfo>()
                        for (item in it.data as List<*>) {
                            val json = gson.toJson(item)
                            val info = JsonUtils.fromJson<FileInfo>(json)
                            if (info != null) {
                                list.add(info)
                            }
                        }
                        if (list.size == 1) {
                            val info = list[0]
                            val downLoadName =
                                "student-${LoginConfiguration.userInfo!!.user.id}.docx"
                            println("download:${info.path}")
                            FileDownloadUtil.downLoad(info.path, downLoadName) {
                                Toast.makeText(
                                    this@SignInFragment.context,
                                    "文件下载完成",
                                    Toast.LENGTH_SHORT
                                ).show()
                            }
                        }
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            }

            val data = AdminQueryStudentStatistic(LoginConfiguration.userInfo!!.user.id)
            service.generateStudentStatistic(LoginConfiguration.userInfo!!.token, data)
                .enqueue(handler)

        }
    }


    private fun loadStatisticInfo() {
        val service = ServiceCreator.create(CommonService::class.java)
        val handler = object : Callback<Result> {
            override fun onResponse(call: Call<Result>, response: Response<Result>) {
                ResponseHandler.handle(response, {
                    val gson = Gson()
                    val list = ArrayList<StatisticInfo>()
                    println(gson.toJson(it))
                    for (item in it.data as List<*>) {
                        val json = gson.toJson(item)
                        val info = JsonUtils.fromJson<StatisticInfo>(json)
                        if (info != null) {
                            list.add(info)
                        }
                    }
                    renderStatisticInfoList(list)
                }, {
                    ResponseHandler.showMessage(it)
                })
            }

            override fun onFailure(call: Call<Result>, t: Throwable) {
                t.printStackTrace()
            }
        }

        val data = AdminQueryStudentStatistic(LoginConfiguration.userInfo!!.user.id)
        service.queryStudentStatistic(LoginConfiguration.userInfo!!.token, data).enqueue(handler)

    }

    private fun renderStatisticInfoList(list: List<StatisticInfo>) {
        var first = true
        val nameBuilder = StringBuilder()
        val rateBuilder = StringBuilder()

        for (item in list) {
            if (!first) {
                nameBuilder.append(",")
                rateBuilder.append(",")
            }
            nameBuilder.append(item.course.name)
            rateBuilder.append(item.rate * 100)
            first = false
        }
        val chartOptions = """
            setChartOptions(
                {
                    xAxis: {type: 'value'}, 
                    yAxis: {type: 'category', data: [${nameBuilder}]},
                    series: [{
                        type: 'bar',
                        data: [${rateBuilder}],
                        label: {show: true, position: 'inside'}
                    }]
                }
                );
                """
        println(chartOptions)
        binding.backupChart.evaluateJavascript(chartOptions, null)
    }

}