package org.brynhild.graduation.activity.user.ui.timetable

import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.PopupMenu
import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import androidx.fragment.app.Fragment
import com.google.gson.Gson
import com.zhuangfei.timetable.listener.ISchedule
import com.zhuangfei.timetable.listener.IWeekView
import com.zhuangfei.timetable.listener.OnSlideBuildAdapter
import com.zhuangfei.timetable.model.Schedule
import org.brynhild.graduation.R
import org.brynhild.graduation.common.config.LoginConfiguration
import org.brynhild.graduation.common.config.MyApplication
import org.brynhild.graduation.common.transfer.Result
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.timetable.QueryTimeTableRequest
import org.brynhild.graduation.common.utils.JsonUtils
import org.brynhild.graduation.databinding.FragmentTimetableBinding
import org.brynhild.graduation.network.entiity.TimeTableInfo
import org.brynhild.graduation.service.http.StudentService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

class TimetableFragment : Fragment() {

    private var _binding: FragmentTimetableBinding? = null
    private val subjects = ArrayList<TimeTableInfo>()
    private val binding get() = _binding!!

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {

        _binding = FragmentTimetableBinding.inflate(inflater, container, false)
        val root: View = binding.root

        binding.idMore.setOnClickListener {
            showPopMenu()
        }

        binding.idLayout.setOnClickListener {
            when (it.id) {
                R.id.id_layout -> {
                    if (binding.idWeekview.isShowing) {
                        binding.idWeekview.isShow(false)
                        binding.idTitle.setTextColor(resources.getColor(com.zhuangfei.android_timetableview.sample.R.color.app_course_textcolor_blue))
                        val cur = binding.idTimetableView.curWeek()
                        binding.idTimetableView.onDateBuildListener()
                            .onUpdateDate(cur, cur)
                        binding.idTimetableView.changeWeekOnly(cur)
                    } else {
                        binding.idWeekview.isShow(true)
                        binding.idTitle.setTextColor(resources.getColor(com.zhuangfei.android_timetableview.sample.R.color.app_red))
                    }
                }
            }
        }


        LoginConfiguration.loadTimeTableConfig {
            initTimetableView()
            loadTimeTableData()
        }

        return root
    }

    private fun loadTimeTableData() {
        val service = ServiceCreator.create(StudentService::class.java)
        val data = QueryTimeTableRequest(
            null,
            null,
            null,
            null,
            null,
            null,
            "${LoginConfiguration.userInfo!!.user.departmentId}",
            null,
            null,
            null
        )
        service.queryTimeTable(LoginConfiguration.userInfo!!.token, data)
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, {
                        val gson = Gson()
                        for (item in it.data as ArrayList<*>) {
                            val json = gson.toJson(item)
                            val timetable = JsonUtils.fromJson<TimeTableInfo>(json)
                            if (timetable != null) {
                                subjects.add(timetable)
                            }
                        }
                        initTimetableView()
                    }, {
                        ResponseHandler.showMessage(it)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })

    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }

    override fun onStart() {
        super.onStart()
        binding.idTimetableView.onDateBuildListener()
            .onHighLight()
    }

    private fun display(list: List<Schedule>) {
        val builder = StringBuilder()
        for (item in list) {
            builder.append(item.name).append("、")
        }
        Toast.makeText(MyApplication.context, builder.toString(), Toast.LENGTH_SHORT).show()
    }

    private fun showPopMenu() {
        val menu = PopupMenu(this@TimetableFragment.context, binding.idMore)
        menu.menuInflater.inflate(R.menu.timetable_menu, menu.menu)
        menu.setOnMenuItemClickListener {
            when (it.itemId) {
                R.id.showAllCourse -> {
                    binding.idTimetableView.isShowNotCurWeek(true).updateView()
                }
                R.id.showCurrentCourse -> {
                    binding.idTimetableView.isShowNotCurWeek(false).updateView()
                }
                R.id.showTime -> {
                    val time = mutableListOf(
                        "08:00",
                        "08:55",
                        "10:10",
                        "11:05",
                        "14:00",
                        "14:55",
                        "16:10",
                        "17:05",
                        "19:00",
                        "19:55",
                        "20:50"
                    ).toTypedArray()
                    val listener =
                        binding.idTimetableView.onSlideBuildListener() as OnSlideBuildAdapter
                    listener.times = time
                    listener.setTimeTextColor(Color.BLACK)
                    binding.idTimetableView.updateSlideView()
                }
                R.id.hiddenTime -> {
                    binding.idTimetableView.callback(OnSlideBuildAdapter())
                    binding.idTimetableView.updateSlideView()
                }
            }
            true
        }
        menu.show()
    }

    private fun initTimetableView() {
        with(binding.idWeekview) {
            source(subjects)
            callback { week ->
                val cur = binding.idTimetableView.curWeek()
                binding.idTimetableView.onDateBuildListener()
                    .onUpdateDate(cur, week)
                binding.idTimetableView.changeWeekOnly(week)
            }
            callback(IWeekView.OnWeekLeftClickedListener {
                val itemCount = binding.idWeekview.itemCount()
                val items = mutableListOf<String>()
                for (i in 1..itemCount) {
                    items.add("第${i}周")
                }
                var target = -1

                if (this@TimetableFragment.context != null) {
                    val builder = AlertDialog.Builder(this@with.context)
                    with(builder) {
                        binding.idTitle.text = "设置当前周"
                        setSingleChoiceItems(
                            items.toTypedArray(), binding.idTimetableView.curWeek() - 1
                        ) { _, p1 -> target = p1 }
                        setPositiveButton(
                            "设置为当前周"
                        ) { _, _ ->
                            if (target != -1) {
                                binding.idWeekview.curWeek(target + 1).updateView()
                                binding.idTimetableView.changeWeekForce(target + 1)
                            }
                        }
                        setNegativeButton("取消", null)
                        create()
                    }.show()
                }

            })
            isShow(false)
        }.showView()


        with(binding.idTimetableView) {
            source(subjects)
            curWeek(LoginConfiguration.timetableConfig!!.week)
            curTerm("一个学期罢了")
            maxSlideItem(11)
            monthWidthDp(50)
            callback(ISchedule.OnItemClickListener { v, scheduleList ->
                display(scheduleList)
            })
            callback(ISchedule.OnItemLongClickListener { v, day, start ->
                Toast.makeText(MyApplication.context, "长按：周${day}，第${start}节", Toast.LENGTH_SHORT)
                    .show()
            })
            callback(ISchedule.OnWeekChangedListener {
                binding.idTitle.text = "第${it}周"
            })
            callback(ISchedule.OnFlaglayoutClickListener { day, start ->
                binding.idTimetableView.hideFlaglayout()
            })
        }.showView()

//        val time= mutableListOf("08:00","08:55","10:10","11:05","14:00","14:55","16:10","17:05","19:00","19:55","20:50").toTypedArray()
        val listener = binding.idTimetableView.onSlideBuildListener() as OnSlideBuildAdapter
        listener.times = LoginConfiguration.timetableConfig!!.times.toTypedArray()
        listener.setTimeTextColor(Color.BLACK)
        binding.idTimetableView.updateSlideView()
    }
}