package org.brynhild.graduation.common.config

import android.content.Context
import android.content.Intent
import android.util.Log
import android.widget.Toast
import com.google.gson.Gson
import org.brynhild.graduation.common.callback.DefaultCallback
import org.brynhild.graduation.common.constant.BroadcastConstant
import org.brynhild.graduation.common.transfer.Result
import org.brynhild.graduation.common.transfer.bo.timetable.TimeTableConfig
import org.brynhild.graduation.common.transfer.dto.UserInfo
import org.brynhild.graduation.common.transfer.handler.ResponseHandler
import org.brynhild.graduation.common.transfer.utils.ServiceCreator
import org.brynhild.graduation.common.transfer.vo.user.AccountLoginInfo
import org.brynhild.graduation.common.utils.JsonUtils
import org.brynhild.graduation.service.http.AccountService
import org.brynhild.graduation.service.http.CommonService
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response

object LoginConfiguration {

    var login_username: String? = null
    var login_password: String? = null
    var userInfo: UserInfo? = null
    var timetableConfig: TimeTableConfig? = null

    fun isValid(): Boolean {
        return login_username?.trim()?.isNotBlank() ?: false && login_password?.trim()
            ?.isNotBlank() ?: false
    }

    fun login() {
        val accountService = ServiceCreator.create(AccountService::class.java)
        val loginInfo =
            login_password?.let { login_username?.let { it1 -> AccountLoginInfo(it1, it) } }
        if (loginInfo != null) {
            accountService.login(loginInfo).enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response, { result ->
                        Log.d(
                            "LoginConfiguration",
                            "flag is ${result.flag},message is ${result.message},token is ${result.data}"
                        )
                        val json = Gson().toJson(result.data)

                        userInfo = JsonUtils.fromJson<UserInfo>(json)
                        val b = Intent(BroadcastConstant.LOGIN_SUCCESS)
                        MyApplication.context.sendBroadcast(b)
                    }, { result ->
                        Log.d(
                            "LoginConfiguration",
                            "flag is ${result.flag},message is ${result.message}"
                        )
                        Toast.makeText(MyApplication.context, result.message, Toast.LENGTH_SHORT)
                            .show()
                        val b = Intent(BroadcastConstant.LOGIN_FAILED)
                        MyApplication.context.sendBroadcast(b)
                    }, {
                        val b = Intent(BroadcastConstant.LOGIN_FAILED)
                        MyApplication.context.sendBroadcast(b)
                    }, {
                        val b = Intent(BroadcastConstant.LOGIN_FAILED)
                        MyApplication.context.sendBroadcast(b)
                    })
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                    val b = Intent(BroadcastConstant.LOGIN_FAILED)
                    MyApplication.context.sendBroadcast(b)
                }
            })
        }
    }

    fun saveLoginInfo() {
        val accountName = login_username
        val accountPassword = login_password
        val sp = MyApplication.context.getSharedPreferences("account", Context.MODE_PRIVATE)
        with(sp.edit()) {
            putString("accountName", accountName)
            putString("accountPassword", accountPassword)
            apply()
        }
        Log.d("LoginActivity", "save info:name:$accountName,password:$accountPassword")
    }

    fun loadTimeTableConfig(callback: DefaultCallback) {
        val service = ServiceCreator.create(CommonService::class.java)
        service.loadTimeTableConfig()
            .enqueue(object : Callback<Result> {
                override fun onResponse(call: Call<Result>, response: Response<Result>) {
                    ResponseHandler.handle(response) {
                        val gson = Gson()
                        val json = gson.toJson(it.data)
                        timetableConfig = JsonUtils.fromJson<TimeTableConfig>(json)
                        callback.execute()

                    }
                }

                override fun onFailure(call: Call<Result>, t: Throwable) {
                    t.printStackTrace()
                }
            })

    }
}