package org.brynhild.graduation.common.transfer.handler

import android.widget.Toast
import androidx.appcompat.app.AlertDialog
import org.brynhild.graduation.common.config.MyApplication
import org.brynhild.graduation.common.transfer.Result
import retrofit2.Response

object ResponseHandler {
    fun handle(
        response: Response<Result>,
        returnTrue: (Result) -> Unit,
        returnFalse: (Result) -> Unit,
        emptyBody: () -> Unit,
        failed: () -> Unit
    ) {
        if (response.isSuccessful) {
            val result = response.body()
            if (result != null) {
                if (result.flag) {
                    returnTrue(result)
                } else {
                    returnFalse(result)
                }
            } else {
                emptyBody()
            }
        } else {
            failed()
        }
    }

    fun handle(
        response: Response<Result>,
        returnTrue: (Result) -> Unit,
        returnFalse: (Result) -> Unit,
        emptyBody: () -> Unit
    ) {
        handle(response, returnTrue, returnFalse, emptyBody) {
            ignoreWhenFailed()
        }
    }

    fun handle(
        response: Response<Result>,
        returnTrue: (Result) -> Unit,
        returnFalse: (Result) -> Unit
    ) {
        handle(response, returnTrue, returnFalse) {
            ignoreWhenEmpty()
        }
    }

    fun handle(response: Response<Result>, returnTrue: (Result) -> Unit) {
        handle(response, returnTrue) {
            ignoreWhenReturnFalse()
        }
    }

    fun handle(response: Response<Result>) {
        handle(response) {
            ignoreWhenReturnTrue()
        }
    }

    fun showWarningDialogWhenFailed() {
        AlertDialog.Builder(MyApplication.context).apply {
            setTitle("发生错误")
            setMessage("连接服务器超时，请检查网络连接")
            setCancelable(false)
            setPositiveButton("OK") { _, _ ->
            }
            show()
        }
    }

    fun showWarningDialogWhenEmpty() {
        AlertDialog.Builder(MyApplication.context).apply {
            setTitle("发生错误")
            setMessage("服务器未返回数据")
            setCancelable(false)
            setPositiveButton("OK") { _, _ ->
            }
            show()
        }
    }

    fun showWarningDialogWhenFalse() {
        AlertDialog.Builder(MyApplication.context).apply {
            setTitle("发生错误")
            setMessage("操作未完成")
            setCancelable(false)
            setPositiveButton("OK") { _, _ ->
            }
            show()
        }
    }

    fun ignoreWhenFailed() {}
    fun ignoreWhenEmpty() {}
    fun ignoreWhenReturnFalse() {}
    fun ignoreWhenReturnTrue() {}
    fun showMessage(result: Result) {
        Toast.makeText(MyApplication.context, result.message, Toast.LENGTH_SHORT).show()
    }
}