package org.brynhild.graduation.common.utils;

import com.github.abel533.echarts.axis.CategoryAxis;
import com.github.abel533.echarts.axis.ValueAxis;
import com.github.abel533.echarts.code.Trigger;
import com.github.abel533.echarts.code.X;
import com.github.abel533.echarts.json.GsonOption;
import com.github.abel533.echarts.series.Line;

import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

public class EChartOptionUtil {

    public static GsonOption getOptions(String title, String[] xAxis, List<LineWrapper> lineWrappers) {
        List<Line> lines = new ArrayList<>();
        for (LineWrapper wrapper : lineWrappers) {
            Line line = new Line();
            line.smooth(true).name(wrapper.name).data(wrapper.data).itemStyle().normal().lineStyle().shadowColor("rgba(0,0,0,0.4)");
            lines.add(line);
        }
        return getOptionsByLine(title, xAxis, lines);
    }

    private static GsonOption getOptionsByLine(String title, @NotNull String[] xAxis, @NotNull List<Line> lines) {
        GsonOption option = new GsonOption();
        option.title().setText(title);
        List<String> m = new ArrayList<>();
        for (Line line : lines) {
            m.add(line.name());
        }

        option.legend().data(m).x(X.center);
        option.tooltip().trigger(Trigger.axis);
        ValueAxis valueAxis = new ValueAxis();
        option.yAxis(valueAxis);
        CategoryAxis categorxAxis = new CategoryAxis();
        categorxAxis.axisLine().onZero(false);
        categorxAxis.boundaryGap(false);
        categorxAxis.data((Object) xAxis);
        option.xAxis(categorxAxis);

        for (Line line : lines) {
            option.series(line);
        }

        return option;
    }

    public static class LineWrapper {
        public String name;
        public Object[] data;

        public LineWrapper(String name, Object[] data) {
            this.name = name;
            this.data = data;
        }

        public LineWrapper(String name, List<Object> data) {
            this.name = name;
            this.data = data.toArray();
        }
    }
}
