package com.example.devicemanager.view

import android.annotation.SuppressLint
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.LinearLayout
import android.widget.PopupWindow
import org.brynhild.graduation.R
import org.brynhild.graduation.databinding.ConfirmPopupWindowBinding

@SuppressLint("InflateParams")
class ConfirmPopupWindow(context: Context, builder: ConfirmPopupWindowBuilder? = null) :
    PopupWindow() {

    var confirmPopupWindowBinding: ConfirmPopupWindowBinding

    init {
        val inflater = LayoutInflater.from(context)
        confirmPopupWindowBinding = ConfirmPopupWindowBinding.inflate(inflater)
        contentView = confirmPopupWindowBinding.root
        this.width = LinearLayout.LayoutParams.MATCH_PARENT //父布局减去padding
        this.height = LinearLayout.LayoutParams.MATCH_PARENT
        this.animationStyle = R.style.pop_animation  //进入和退出动画效果
        this.isOutsideTouchable = true //是否可以
        this.isClippingEnabled = false //背景透明化可以铺满全屏
        // 设置最终的背景,也可以通过context.resources.getColor(resId)设置自己的颜色
        val colorDrawable = ColorDrawable(Color.parseColor("#00000000"))
        this.setBackgroundDrawable(colorDrawable) //设置背景

        //设置取消的点击事件
        confirmPopupWindowBinding.tvConfirmCancel.setOnClickListener {
            //先隐藏弹窗
            dismiss()
            //调用接口的具体实现
            builder?.mCancelListener?.invoke()
        }

        //设置确认的点击事件
        confirmPopupWindowBinding.tvConfirmEnsure.setOnClickListener {
            //先隐藏弹窗
            dismiss()
            builder?.mConfirmListener?.invoke()
        }
    }

    //创建ConfirmPopupWindow的一个内部类
    //返回Builder就可以流水线定义
    class ConfirmPopupWindowBuilder(val context: Context) {
        companion object {
            fun init(context: Context): ConfirmPopupWindowBuilder {
                return ConfirmPopupWindowBuilder(context)
            }
        }

        var mCancelListener: () -> Unit = {} //一个不需要参数的无返回值的函数
        var mConfirmListener: () -> Unit = {}

        private val window: ConfirmPopupWindow = ConfirmPopupWindow(context, this)

        fun build(): ConfirmPopupWindow {
            window.confirmPopupWindowBinding.tvConfirmContent.visibility = View.VISIBLE
            window.confirmPopupWindowBinding.tvConfirmCancel.visibility = View.VISIBLE
            window.confirmPopupWindowBinding.tvConfirmEnsure.visibility = View.VISIBLE
            return window
        }

        fun setContent(content: String): ConfirmPopupWindowBuilder {
            window.confirmPopupWindowBinding.tvConfirmContent.visibility = View.VISIBLE
            window.confirmPopupWindowBinding.tvConfirmContent.text = content
            return this
        }

        fun setCancelListener(callback: () -> Unit): ConfirmPopupWindowBuilder {
            mCancelListener = callback
            return this
        }

        fun setEnsureListener(callback: () -> Unit): ConfirmPopupWindowBuilder {
            mConfirmListener = callback
            return this
        }

        fun setEnsureText(content: String): ConfirmPopupWindowBuilder {
            window.confirmPopupWindowBinding.tvConfirmEnsure.visibility = View.VISIBLE
            window.confirmPopupWindowBinding.tvConfirmEnsure.text = content
            return this
        }

        fun setCancelText(content: String): ConfirmPopupWindowBuilder {
            window.confirmPopupWindowBinding.tvConfirmCancel.visibility = View.VISIBLE
            window.confirmPopupWindowBinding.tvConfirmCancel.text = content
            return this
        }

    }

    fun show() {
        showAtLocation(contentView, Gravity.CENTER, 0, 0)
    }
}