package org.brynhild.graduation.network.adapter

import android.content.Context
import android.content.Intent
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.RecyclerView
import com.google.gson.Gson
import org.brynhild.graduation.R
import org.brynhild.graduation.activity.admin.DeviceDetailInfoActivity
import org.brynhild.graduation.common.config.MyApplication
import org.brynhild.graduation.common.constant.DeviceConstant
import org.brynhild.graduation.network.entiity.Device

class DeviceAdapter(private val context: Context, private val deviceList: List<Device>) :
    RecyclerView.Adapter<DeviceAdapter.ViewHolder>() {

    inner class ViewHolder(view: View) : RecyclerView.ViewHolder(view) {
        val name: TextView = view.findViewById(R.id.name)
        val mac: TextView = view.findViewById(R.id.mac)
        val type: TextView = view.findViewById(R.id.type)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): ViewHolder {
        val view = LayoutInflater.from(context).inflate(R.layout.device_item, parent, false)
        val holder = ViewHolder(view)
        holder.itemView.setOnClickListener {
            val position = holder.adapterPosition
            val classEntity = deviceList[position]
            val intent = Intent(MyApplication.context, DeviceDetailInfoActivity::class.java)
            intent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
            val gson = Gson()
            intent.putExtra("device", gson.toJson(classEntity))
            MyApplication.context.startActivity(intent)
        }
        return holder
    }

    override fun onBindViewHolder(holder: ViewHolder, position: Int) {
        val user = deviceList[position]
        holder.name.text = user.name
        holder.mac.text = user.mac
        when (user.type) {
            DeviceConstant.PRIMARY_GATEWAY -> {
                holder.type.text = "一级网关"
            }
            DeviceConstant.SECONDARY_GATEWAY -> {
                holder.type.text = "二级网关"
            }
            DeviceConstant.TERMINAL -> {
                holder.type.text = "终端机"
            }
            else -> {
                holder.type.text = "错误的设备类型"
            }
        }
    }

    override fun getItemCount() = deviceList.size
}