package org.brynhild.graduation.service.mqtt;


import org.brynhild.graduation.common.config.MqttServerConfig;
import org.eclipse.paho.client.mqttv3.MqttClient;
import org.eclipse.paho.client.mqttv3.MqttConnectOptions;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.eclipse.paho.client.mqttv3.persist.MemoryPersistence;

// 订阅和接受消息
public class SubscribeClient {


    // 连接客户端
    private final MqttClient client;
    private String clientId = "subscribe_client";


    // 构造方法，实例化客户端并连接
    public SubscribeClient(String clientId) throws MqttException {

        this.clientId = clientId;

        // 构建包含连接参数的连接选择对象
        MqttConnectOptions options = new MqttConnectOptions();
        // 设置Mqtt版本
        options.setMqttVersion(MqttConnectOptions.MQTT_VERSION_3_1_1);
        // 设置清空Session，false表示服务器会保留客户端的连接记录，true表示每次以新的身份连接到服务器
        options.setCleanSession(true);
        // 设置超时时间，单位为秒
        options.setConnectionTimeout(30);
        // 设置会话心跳时间，单位为秒，客户端每隔10秒向服务端发送心跳包判断客户端是否在线
        options.setKeepAliveInterval(10);
        // 客户端是否自动尝试重新连接到服务器
        options.setAutomaticReconnect(true);

        options.setUserName("client");
        options.setPassword("client".toCharArray());

        // 创建MQTT客户端并发起连接代理服务器
        System.out.println("注册新的clientId:" + clientId);
        client = new MqttClient(MqttServerConfig.broker, clientId, new MemoryPersistence());
        // 设置回调
        client.setCallback(new PushCallback(this.clientId));
        // 发起连接
        client.connect(options);
    }


    // 订阅指定主题
    public void subscribe() throws MqttException {
        System.out.println("MQTT subscribe on: " + MqttServerConfig.INSTANCE.getListenTopic());
        client.subscribe(MqttServerConfig.INSTANCE.getListenTopic(), MqttServerConfig.qos);
    }

    public void publish(String content) throws MqttException {
        // 定义消息：hello
        MqttMessage message = new MqttMessage();
        message.setQos(MqttServerConfig.qos);
        message.setPayload(content.getBytes());
        // 发布消息
        client.publish(MqttServerConfig.registerTopic, message);
    }

}
